﻿local addOn, ns = ...
local L = ns.L

local table_sort = table.sort

local GetItemInfo = GetItemInfo


local events = CreateFrame('Frame')
events:SetScript('OnEvent', function(self, event, ...)
	self[event](self, event, ...)
end)
events:RegisterEvent("LOOT_SLOT_CLEARED");
events:RegisterEvent('UI_ERROR_MESSAGE');

local sortingButtons = {}
local default_sorting_methode = 3

local activeLoot = {}
local activeLootData = {}

ns.Slot_QueueList = {}

local mover = CreateFrame("Frame", nil, UIParent)
--mover:EnableMouse(true)
mover:SetMovable(true)
mover:RegisterForDrag("LeftButton")
mover:SetPoint("CENTER", UIParent, "CENTER", 0,0)
mover:SetSize(328, 28)
--[[
mover.bg = mover:CreateTexture()
mover.bg:SetAllPoints()
mover.bg:SetTexture(0,0,0,0.6)
]]
mover:SetScript("OnDragStart", function()
	ns:StartLootRollsSideMoving()
end)
mover:SetScript("OnDragStop", function()
	ns:StopLootRollsSideMoving()
end)
--[[
mover.text = mover:CreateFontString()
mover.text:SetPoint("CENTER", mover, "CENTER", 0,0)
mover.text:SetFont(ns.default_font, 10, 'NONE')
mover.text:SetText('LootRolls Mover')
]]
function ns:StartLootRollsSideMoving()
	mover:StartMoving()
end
function ns:StopLootRollsSideMoving()
	mover:StopMovingOrSizing()	
	local x, y = mover:GetCenter()
	local ux, uy = UIParent:GetCenter()
	ns.db.LootRollMover = { floor(x - ux + 0.5),floor(y - uy + 0.5) }
	ns:UpdateLootRollsSizePosition()
end
function ns:UpdateLootRollsSizePosition()
	mover:SetPoint("CENTER", UIParent, "CENTER", ns.db.LootRollMover[1] or 0, ns.db.LootRollMover[2] or 0)
end

do
	local lastNumRolled = 0
	local lastText = ''
	local lastText2 = ''
	local rolledList = {}
	
	function ns.GetRolledPlayersLists()

		if ns:GetTotalListRollsLen() ~= lastNumRolled then
			lastNumRolled = ns:GetTotalListRollsLen()
			lastText = ''
			lastText2 = ''
			wipe(rolledList)
			
			for index=1, lastNumRolled do
				
				local name, data, coloredName = ns.GetPlayerRolledInfo(index)
				
				local temp_name = Ambiguate(name, "short")
				
				rolledList[temp_name] = true
				
				if coloredName then
					if index == lastNumRolled then
						lastText = lastText..coloredName
					else
						lastText = lastText..coloredName ..', '
					end
				else
					if index == lastNumRolled then
						lastText = lastText..temp_name
					else
						lastText = lastText..temp_name..', '
					end
				end
			end
			
			
			for index=1, GetNumGroupMembers() do
				local unit = 'raid'..index
				local name = UnitName(unit)
			
				if not rolledList[name] then
					local _, classFile = UnitClass(unit)
					local className, classFile, index, colorStr = ns.GetClassInfo( classFile )
					
					if index == GetNumGroupMembers() then
						lastText2 = lastText2..colorStr..name
					else
						lastText2 = lastText2..'|c'..colorStr..name..'|r, '
					end
				end
			end
		end
		
		
		return lastText, lastText2
	end
	
	
end


local rolledTooltipPattern = '|cFF00FF00Готово|r: %s\n\n|cFFFF0000Ожидание|r: %s'
local itemRolls = {}

local MainFrame = CreateFrame('Frame', addOn..'MainFrame', UIParent); ns.LootRollsMainFrame = MainFrame
MainFrame:SetFrameStrata('MEDIUM')
MainFrame:SetFrameLevel(15)
MainFrame.sorting = default_sorting_methode
MainFrame:SetPoint('TOP', mover, 'BOTTOM', 0, 0)
MainFrame:SetSize(600, 310)

MainFrame.width = 600
MainFrame.height = 310

MainFrame.width_min = 600
MainFrame.height_min = 40

MainFrame.bg = MainFrame:CreateTexture(nil, 'BACKGROUND')
MainFrame.bg:SetAllPoints()
ns.CustomSetTexture(MainFrame.bg, 0,0,0,0.7)
MainFrame.icons = {}
MainFrame:Hide()
MainFrame:EnableMouse(true)
MainFrame:RegisterForDrag("LeftButton")
MainFrame:SetScript("OnDragStart", function() ns:StartLootRollsSideMoving() end)
MainFrame:SetScript("OnDragStop", function() ns:StopLootRollsSideMoving() end)
MainFrame:SetScript("OnUpdate", function(self, elapsed)
	self._elapsed = ( self._elapsed or 0 ) + elapsed
	if self._elapsed < 0.5 then return end
	self._elapsed = 0
	
	local timer = max(0, ( self.endTime or 0 ) - GetTime() )
	
	if self.minimize.toggle and MainFrame.setCurrentRollID and activeLootData[MainFrame.setCurrentRollID] then
		self.infoHeader:SetText(format('%d/%d - %dc', ns:GetTotalListRollsLen(), GetNumGroupMembers(), timer))
		
		if self.infoHeader.tooltip:IsShown() then
			self.infoHeader.tooltip.text:SetText(format(rolledTooltipPattern, ns.GetRolledPlayersLists()))
		end
	else
		self.infoHeader:SetText(format('%dc', timer))
	end
end)

MainFrame.infoHeader = MainFrame:CreateFontString(nil, 'ARTWORK')
MainFrame.infoHeader:SetPoint("TOP", MainFrame, "TOP", 0 , -3)
MainFrame.infoHeader:SetTextColor(1, 1, 1)
MainFrame.infoHeader:SetJustifyH("CENTER")
MainFrame.infoHeader:SetWordWrap(false)
MainFrame.infoHeader:SetFont(ns.default_font, ns.default_font_size+1, 'NONE')
MainFrame.infoHeader:SetShadowColor(0,0,0,1)
MainFrame.infoHeader:SetShadowOffset(1,-1)
MainFrame.infoHeader:SetText('TestHeader')

MainFrame.infoHeader.mouseOver = CreateFrame("Frame", nil, MainFrame)
MainFrame.infoHeader.mouseOver:SetSize(1,1)
MainFrame.infoHeader.mouseOver:SetAllPoints(MainFrame.infoHeader)
MainFrame.infoHeader.mouseOver:SetFrameStrata('MEDIUM')
MainFrame.infoHeader.mouseOver:EnableMouse(true)
MainFrame.infoHeader.mouseOver:SetScript('OnEnter', function()
	if MainFrame.minimize.toggle and MainFrame.setCurrentRollID then
--		print('MainFrame.infoHeader.mouseOver', 'OnEnter')
		MainFrame._elapsed = 1
		MainFrame.infoHeader.tooltip:Show()
	end
end)
MainFrame.infoHeader.mouseOver:SetScript('OnLeave', function()
--	print('MainFrame.infoHeader.mouseOver', 'OnLeave')
	MainFrame.infoHeader.tooltip:Hide()
end)

MainFrame.infoHeader.tooltip = CreateFrame("Frame", nil, MainFrame)
MainFrame.infoHeader.tooltip:SetPoint('TOP', MainFrame.infoHeader.mouseOver, 'BOTTOM', 0, -2)
MainFrame.infoHeader.tooltip:SetSize(300, 150)
MainFrame.infoHeader.tooltip:SetFrameStrata('TOOLTIP')
MainFrame.infoHeader.tooltip:Hide()

MainFrame.infoHeader.tooltip.text = MainFrame.infoHeader.tooltip:CreateFontString(nil, 'ARTWORK')
MainFrame.infoHeader.tooltip.text:SetPoint("TOP", MainFrame.infoHeader.tooltip, "TOP", 0 , -3)
MainFrame.infoHeader.tooltip.text:SetTextColor(1, 1, 1)
MainFrame.infoHeader.tooltip.text:SetJustifyH("LEFT")
MainFrame.infoHeader.tooltip.text:SetJustifyV("TOP")
MainFrame.infoHeader.tooltip.text:SetWordWrap(true)
MainFrame.infoHeader.tooltip.text:SetFont(ns.default_font, ns.default_font_size+1, 'NONE')
MainFrame.infoHeader.tooltip.text:SetShadowColor(0,0,0,1)
MainFrame.infoHeader.tooltip.text:SetShadowOffset(1,-1)
MainFrame.infoHeader.tooltip.text:SetText('TestHeader')
MainFrame.infoHeader.tooltip.text:SetWidth(300)

MainFrame.infoHeader.tooltip.bg = MainFrame.infoHeader.tooltip:CreateTexture(nil, 'BACKGROUND')
MainFrame.infoHeader.tooltip.bg:SetPoint('TOPLEFT', MainFrame.infoHeader.tooltip, 'TOPLEFT', -2, 2)
MainFrame.infoHeader.tooltip.bg:SetPoint('BOTTOMRIGHT', MainFrame.infoHeader.tooltip.text, 'BOTTOMRIGHT', 2, -2)
ns.CustomSetTexture(MainFrame.infoHeader.tooltip.bg, 0,0,0,0.7)

MainFrame.left = MainFrame:CreateTexture(nil, 'OVERLAY')
MainFrame.left:SetPoint('TOPLEFT', MainFrame, 'TOPLEFT', 0, 0)
MainFrame.left:SetPoint('BOTTOMLEFT', MainFrame, 'BOTTOMLEFT', 0, 0)
MainFrame.left:SetSize(1,1)
ns.CustomSetTexture(MainFrame.left, 0,0,0,1)

MainFrame.right = MainFrame:CreateTexture(nil, 'OVERLAY')
MainFrame.right:SetPoint('TOPRIGHT', MainFrame, 'TOPRIGHT', 0, 0)
MainFrame.right:SetPoint('BOTTOMRIGHT', MainFrame, 'BOTTOMRIGHT', 0, 0)
MainFrame.right:SetSize(1,1)
ns.CustomSetTexture(MainFrame.right, 0,0,0,1)

MainFrame.bottom = MainFrame:CreateTexture(nil, 'OVERLAY')
MainFrame.bottom:SetPoint('BOTTOMLEFT', MainFrame, 'BOTTOMLEFT', 0, 0)
MainFrame.bottom:SetPoint('BOTTOMRIGHT', MainFrame, 'BOTTOMRIGHT', 0, 0)
MainFrame.bottom:SetSize(1,1)
ns.CustomSetTexture(MainFrame.bottom, 0,0,0,1)

MainFrame.top = MainFrame:CreateTexture(nil, 'OVERLAY')
MainFrame.top:SetPoint('TOPLEFT', MainFrame, 'TOPLEFT', 0, 0)
MainFrame.top:SetPoint('TOPRIGHT', MainFrame, 'TOPRIGHT', 0, 0)
MainFrame.top:SetSize(1,1)
ns.CustomSetTexture(MainFrame.top, 0,0,0,1)

MainFrame.close = CreateFrame("Button", nil, MainFrame, 'UIPanelCloseButton')
MainFrame.close:SetFrameLevel(MainFrame:GetFrameLevel()+5)
MainFrame.close:SetSize(32, 32)	
MainFrame.close:SetPoint("TOPRIGHT", MainFrame, "TOPRIGHT", 3, 3)	

MainFrame.close:SetScript("OnClick", function() 
	MainFrame:Hide()
	MainFrame.scrollFrame.selectedName = nil
	--ns:AddToIgnoreCurrentLootBids()
	ns:AddToTimeOutCurrentLootBids()
end)


MainFrame.minimize = CreateFrame("Button", nil, MainFrame, 'UIPanelCloseButton')
MainFrame.minimize:SetFrameLevel(MainFrame:GetFrameLevel()+5)
MainFrame.minimize:SetSize(32, 32)	
MainFrame.minimize.toggle = true
MainFrame.minimize:SetPoint("RIGHT", MainFrame.close, "LEFT", 10, 0)	
MainFrame.minimize:SetNormalTexture("Interface\\Buttons\\UI-Panel-BiggerButton-Up")
MainFrame.minimize:SetPushedTexture("Interface\\Buttons\\UI-Panel-BiggerButton-Down")
MainFrame.minimize:SetHighlightTexture("Interface\\Buttons\\UI-Panel-MinimizeButton-Highlight")
MainFrame.minimize:SetScript("OnClick", function(self) 
	self.toggle = not self.toggle
	self:GetParent()._elapsed = 1
	
	local normal = self:GetNormalTexture()
	local pushed = self:GetPushedTexture()
	local highlight = self:GetHighlightTexture()
	
--	self.tabShown = MainFrame.selectTab:IsShown()
	
	if self.toggle then
		normal:SetTexCoord(0, 1, 0, 1)
		pushed:SetTexCoord(0, 1, 0, 1)
		highlight:SetTexCoord(0, 1, 0, 1)
		
	--	for i=1, #MainFrame.icons do
	--		if MainFrame.icons[i].rollID then
	--			MainFrame.icons[i]:Show()
	--		end
	--	end
		MainFrame.noteButton.Handler:Show()
		MainFrame.scrollFrame:Show()
		MainFrame:SetSize(MainFrame.width, MainFrame.height)
		MainFrame.selectTab:SetAlpha(1)
		MainFrame.noteButton:Show()
	else
		normal:SetTexCoord(1, 0, 1, 0)
		pushed:SetTexCoord(1, 0, 1, 0)
		highlight:SetTexCoord(1, 0, 1, 0)
		
		
	--	for i=1, #MainFrame.icons do
	--		MainFrame.icons[i]:Hide()		
	--	end
		MainFrame.noteButton.Handler:Hide()
		MainFrame.scrollFrame:Hide()
		MainFrame:SetSize(MainFrame.width_min, MainFrame.height_min)
		MainFrame.selectTab:SetAlpha(0)
		MainFrame.noteButton:Hide()
	end
	
--	print('T', 'MainFrame.minimize', 'Click', self.toggle and 'Open' or 'Hide', normal)
end)

MainFrame.ShowPass = CreateFrame("Frame", nil, MainFrame)
MainFrame.ShowPass:SetSize(120, 20)
MainFrame.ShowPass:SetPoint('RIGHT', MainFrame.close, 'LEFT', 0, 3)
MainFrame.ShowPass.button = CreateFrame('CheckButton', nil, MainFrame.ShowPass, "UICheckButtonTemplate") --"UICheckButtonTemplate"
MainFrame.ShowPass.button:SetFrameLevel(MainFrame.ShowPass:GetFrameLevel() + 1)
MainFrame.ShowPass.button:SetPoint("TOPLEFT")
MainFrame.ShowPass.button:SetSize(20, 20)
MainFrame.ShowPass.button:SetCheckedTexture("Interface\\Buttons\\UI-CheckBox-Check")
MainFrame.ShowPass.button:SetDisabledCheckedTexture("Interface\\Buttons\\UI-CheckBox-Check-Disabled")
MainFrame.ShowPass.button:SetScript("OnClick", function(self)
	ns.db.showPass = not ns.db.showPass
	self:SetChecked(ns.db.showPass)
	MainFrame.scrollFrame.lastTableLen = -1
	ns:UpdateRollsListForRollID( MainFrame.setCurrentRollID )
end)
MainFrame.ShowPass.button:SetScript('OnShow', function(self)
	self:SetChecked(ns.db.showPass)
end)
MainFrame.ShowPass.button:SetScript('OnHide', function(self)
	self:SetChecked(ns.db.showPass)
end)

MainFrame.ShowPass.text = MainFrame.ShowPass:CreateFontString(nil, 'OVERLAY', "GameFontHighlight")
MainFrame.ShowPass.text:SetPoint("LEFT", MainFrame.ShowPass.button, "RIGHT", 0 , 0)
MainFrame.ShowPass.text:SetPoint("RIGHT", MainFrame.ShowPass, "RIGHT", 0 , 0)	
MainFrame.ShowPass.text:SetText(L['GUI_SHOWPASSES'])
MainFrame.ShowPass.text:SetTextColor(1, 1, 1)
MainFrame.ShowPass.text:SetJustifyH("LEFT")
MainFrame.ShowPass.text:SetWordWrap(false)
MainFrame.ShowPass.text:SetFont(ns.default_font, ns.default_font_size, 'NONE')
MainFrame.ShowPass.text:SetShadowColor(0,0,0,1)
MainFrame.ShowPass.text:SetShadowOffset(1,-1)

MainFrame.noteButton = CreateFrame('Button', nil, MainFrame)
MainFrame.noteButton:SetSize(22, 22)
MainFrame.noteButton:SetNormalTexture("Interface/BUTTONS/UI-GuildButton-PublicNote-Up.png")
MainFrame.noteButton:SetPoint('TOPRIGHT', MainFrame, 'TOPRIGHT',  -5, -25)
MainFrame.noteButton:SetScript('OnClick', function()
	ns.ToggleLootListFrame()
end)
MainFrame.noteButton.Handler = CreateFrame('Frame', nil, MainFrame)
MainFrame.noteButton.Handler:SetScript('OnShow', ns.LootList_ResetPosition)

local rollTypeColors = ns.rollTypeColors

local HideTip = ns.HideTip
local HideTip2  = ns.HideTip2
local SetTip = ns.SetTip
local SetItemTip = ns.SetItemTip
local ItemOnUpdate = ns.ItemOnUpdate

local function LootClick(frame)
	if IsControlKeyDown() then DressUpItemLink(frame.link)
	elseif IsShiftKeyDown() then ChatEdit_InsertLink(frame.link) 
	else
--		ns.print('Normal click')
		MainFrame.scrollFrame.selectedName = nil
		frame:GetParent().setCurrentRollID = frame.rollID
		MainFrame.scrollFrame.lastTableLen = -1
		ns:UpdateRollsListForRollID(frame.rollID)
		ns:UpdateColoredFilterSettings()
	end
end
local function ChangeSorting(frame)	
	if not frame.sorting then return end	
	
--	ns.print('T', 'ChangeSorting', MainFrame.sorting ,'->', frame.sorting)

	if MainFrame.sorting == frame.sorting then
		MainFrame.sorting = frame.revsorting
	else
		MainFrame.sorting = frame.sorting
	end
	
	MainFrame.scrollFrame.lastTableLen = -1
	ns:UpdateRollsListForRollID( MainFrame.setCurrentRollID )
end

local function CreateItemIcon(frame)

	local button = CreateFrame("Button")
	button:SetSize(28, 28)
	button:SetParent(frame)
	button:SetFrameLevel(frame:GetFrameLevel()+1)
	
	button:SetScript("OnEnter", SetItemTip)
	button:SetScript("OnLeave", HideTip2)
--	button:SetScript("OnUpdate", ItemOnUpdate)
	button:SetScript("OnClick", LootClick)
	button:SetScript('OnEvent', SetItemTip)
	
	button.icon = button:CreateTexture(nil, 'OVERLAY')
	button.icon:SetSize(28,28)
	button.icon:SetPoint('CENTER', button, 'CENTER', 0, 0)
	button.icon:SetTexCoord(0.07, 0.93, 0.07, 0.93)
	
	local text = button:CreateFontString(nil, "OVERLAY")
	text:SetFont(ns.default_font, ns.default_font_size+2, 'OUTLINE')
	text:SetPoint("BOTTOM", button.icon, "BOTTOM", 0, 0)
	text:SetSize(50, 50)
	text:SetJustifyH("CENTER")
	text:SetJustifyV("BOTTOM")	
	text:SetTextColor(1, 1, 1, 1)
	text:SetShadowColor(0,0,0,1)
	text:SetShadowOffset(1,-1)
	text:SetWordWrap(true)
	
	button.text = text
	
	local numItems = button:CreateFontString(nil, "OVERLAY")
	numItems:SetFont(ns.default_font, ns.default_font_size+2, 'OUTLINE')
	numItems:SetPoint("TOPRIGHT", button.icon, "TOPRIGHT", 0, 0)
	numItems:SetSize(50, 50)
	numItems:SetJustifyH("RIGHT")
	numItems:SetJustifyV("TOP")	
	numItems:SetTextColor(1, 1, 1, 1)
	numItems:SetShadowColor(0,0,0,1)
	numItems:SetShadowOffset(1,-1)
	numItems:SetWordWrap(true)
	
	button.numItems = numItems
	
	button.bg = button:CreateTexture(nil, 'OVERLAY', nil, -1)
	button.bg:SetSize(30,30)
	button.bg:SetPoint('TOPLEFT', button.icon, 'TOPLEFT', -1, 1)
	button.bg:SetPoint('BOTTOMRIGHT', button.icon, 'BOTTOMRIGHT', 1, -1)
	ns.CustomSetTexture(button.bg, 0,0,0,1)
	
	button.addFrame = CreateFrame('Frame', nil, button)
	button.addFrame:SetSize(15, 15)
	button.addFrame:SetPoint('TOPLEFT', button, 'BOTTOMLEFT', 0, 0)
	button.addFrame:SetPoint('TOPRIGHT', button, 'TOPRIGHT', 0, 0)
	
	button.addFrame.text = button.addFrame:CreateFontString(nil, "OVERLAY")
	button.addFrame.text:SetFont(ns.default_font, ns.default_font_size+2, 'OUTLINE')
	button.addFrame.text:SetSize(20, 20)
	button.addFrame.text:SetJustifyH("CENTER")
	button.addFrame.text:SetJustifyV("CENTER")	
	button.addFrame.text:SetTextColor(1, 1, 1, 1)
	button.addFrame.text:SetShadowColor(0,0,0,1)
	button.addFrame.text:SetShadowOffset(1,-1)
	button.addFrame.text:SetWordWrap(true)
	button.addFrame.text:SetText('0')
	button.addFrame.text:SetPoint("CENTER", 0, 0)
	
	button.addFrame.bg = button.addFrame:CreateTexture(nil, 'OVERLAY', nil, -1)
	button.addFrame.bg:SetSize(30,30)
	button.addFrame.bg:SetPoint('TOPLEFT', button.addFrame, 'TOPLEFT', 0, 0)
	button.addFrame.bg:SetPoint('BOTTOMRIGHT', button.addFrame, 'BOTTOMRIGHT', 0, 0)
	ns.CustomSetTexture(button.addFrame.bg, 0,0,0,0.8)
	button.addFrame:Hide()
	
	return button
end

for i=1, 10 do
	MainFrame.icons[i] = CreateItemIcon(MainFrame)
	MainFrame.icons[i]:SetPoint('TOPLEFT', MainFrame, 'TOPLEFT', 20 + ( 39 * ( i - 1 ) ) , -20)
	ns.CustomSetTexture(MainFrame.icons[i].icon, 1, 0, 0, 1)
end

local function GetIconTab(rollID)

	for i=1, #MainFrame.icons do
		if not MainFrame.icons[i].rollID then return MainFrame.icons[i] end
		if MainFrame.icons[i].rollID == rollID then return MainFrame.icons[i] end
	end
end

local function GetUsedIconTab(rollID)
	for i=1, #MainFrame.icons do
		if MainFrame.icons[i].rollID == rollID then return MainFrame.icons[i] end
	end
end

function ns.HaveActiveLootRolls()
	return #activeLoot > 0 and true or false
end

function ns.OpenLootRollsMainFrame() 

	--print('T', ns.HaveActiveLootRolls())
	--[==[
	for k,v in pairs(activeLoot) do
		print('T', k,v)
	end
	]==]
	if ns.HaveActiveLootRolls() then
		MainFrame:Show()
	else
		print("|cFFFFFF00LootHelper: Нет открытых сессий")
	end
end


function ns.UpdateIconGrid_ItemGained(link, frame)
--	ns.print('T', 'UpdateIconGrid_ItemGained', link, frame)

	local name, link1, quality, iLevel, reqLevel, class, subclass, maxStack, equipSlot, texture, vendorPrice = GetItemInfo(link)
	
	local itemInfoText = ''
	
--	print('T3', link, ns.Server_ItemIsGrouped(link), ns.NumericItemLink(link), ns.Server_GetAmountGroupedItems(link))
	if ns.Server_ItemIsGrouped(link) then
		frame.numItems:SetText('x'..ns.Server_GetAmountGroupedItems(link))
	else
		frame.numItems:SetText('')
	end
	
	if ns.CheckForContainTooltip(link1, ITEM_BIND_ON_EQUIP) then
		itemInfoText = itemInfoText..'|cFFFFFF00BoE|r '
	elseif ns.CheckForContainTooltip(link1, ITEM_BIND_ON_PICKUP) then
		itemInfoText = itemInfoText..'|cFFFF0000BoP|r '
--	elseif ns.CheckForContainTooltip(link1, ITEM_BIND_TO_BNETACCOUNT) then
--		itemInfoText = 'BoA '
	end

	
	frame.text:SetText(itemInfoText)
	
	frame.link = link
	frame.icon:SetTexture(texture)
	
	if frame.selectedTab then
		local color = ITEM_QUALITY_COLORS[quality]
		ns.CustomSetTexture(frame.selectedTab.bg, color.r, color.g, color.b, .8 )
	end
end

function ns.UpdateIconGrid_NumItems(rollID)
	if rollID then
		for i=1, #MainFrame.icons do	
			if MainFrame.icons[i].rollID == rollID then	
				if ns.Server_ItemIsGrouped(MainFrame.icons[i].link) then
					MainFrame.icons[i].numItems:SetText('x'..ns.Server_GetAmountGroupedItems(MainFrame.icons[i].link))
				else
					MainFrame.icons[i].numItems:SetText('')
				end
			end
		end	
	else
		for i=1, #MainFrame.icons do	
			if MainFrame.icons[i].rollID then	
				if ns.Server_ItemIsGrouped(MainFrame.icons[i].link) then
					MainFrame.icons[i].numItems:SetText('x'..ns.Server_GetAmountGroupedItems(MainFrame.icons[i].link))
				else
					MainFrame.icons[i].numItems:SetText('')
				end
			end
		end
	end
end

local UpdateIconGrid_FrameIndex = 1

function ns:UpdateMainFramVisability()	
	if UpdateIconGrid_FrameIndex == 1 then
		MainFrame:Hide()
		MainFrame.setCurrentRollID = nil
		MainFrame.scrollFrame.lastTableLen = -1 
	elseif ns:IsMasterLooter() then
		MainFrame:Show()
		MainFrame.endTime = GetTime()+60
		MainFrame._elapsed = 1
	elseif ns.db.ShownMainFrame and ns:IsNotTimeOutGUID() then
		MainFrame:Show()
		MainFrame.endTime = GetTime()+60
		MainFrame._elapsed = 1
	end
end

local function UpdateIconGrid()
	UpdateIconGrid_FrameIndex = 1
	
	for i=1, #MainFrame.icons do
		MainFrame.icons[i].rollID = nil
		MainFrame.icons[i]:Hide()
	end
	
	for i=1, #activeLoot do		
		local rollID = activeLoot[i]
		
		local frame = GetIconTab(rollID)	
		if frame then
			
			frame.rollID = rollID
			frame:ClearAllPoints()
			frame:SetPoint('TOPLEFT', MainFrame, 'TOPLEFT', 30 + ( 39 * ( UpdateIconGrid_FrameIndex - 1 ) ) , -20)	
			frame:Show()
			
			if ns:IsMasterLooter() then			
				frame.addFrame:Show()
			else
				frame.addFrame:Hide()
			end
			
			local link = activeLootData[rollID].link
	
			ns.QueueForItemInfo(link, 'UpdateIconGrid_ItemGained', frame)	
			
			UpdateIconGrid_FrameIndex = UpdateIconGrid_FrameIndex + 1
			
			ns:UpdateRollsListForRollID(rollID)
			ns:UpdateAmountRollID(rollID)
		end
		
	end

	if ns:IsMasterLooter() then		
		MainFrame.height = 325
		MainFrame.scrollFrame:SetPoint('TOPLEFT', MainFrame, 'TOPLEFT', 5, -75)
	else
		MainFrame.height = 310
		MainFrame.scrollFrame:SetPoint('TOPLEFT', MainFrame, 'TOPLEFT', 5, -60)
	end
	
	if MainFrame.minimize.toggle then
		MainFrame:SetSize(MainFrame.width, MainFrame.height)
	else
		MainFrame:SetSize(MainFrame.width_min, MainFrame.height_min)
	end
	
	ns:UpdateMainFramVisability()	
end


MainFrame.scrollFrame = CreateFrame('Frame', nil, MainFrame)
MainFrame.scrollFrame:SetPoint('TOPLEFT', MainFrame, 'TOPLEFT', 5, -60)
MainFrame.scrollFrame:SetPoint('BOTTOMRIGHT', MainFrame, 'BOTTOMRIGHT', -5, 5)
MainFrame.scrollFrame.lines = {}
MainFrame.scrollFrame.lastTableLen = -1
MainFrame.scrollFrame.tempFrom = 1
MainFrame.scrollFrame:EnableMouse(true)
MainFrame.scrollFrame:EnableMouseWheel(1)
MainFrame.scrollFrame:EnableMouse(true)
MainFrame.scrollFrame:RegisterForDrag("LeftButton")
MainFrame.scrollFrame:SetScript("OnDragStart", function()
	ns:StartLootRollsSideMoving()
end)
MainFrame.scrollFrame:SetScript("OnDragStop", function()
	ns:StopLootRollsSideMoving()
end)
MainFrame.scrollFrame:SetScript("OnMouseWheel", function(self, delta)			

	self.tempFrom = self.tempFrom - delta
	
	if self.tempFrom < 1 then
		self.tempFrom = 1
	end
	
	local minFrom = ns:GetSortingTableLen() - 9
	
	if minFrom < 1 then
		minFrom = 1
	end
	
	if self.tempFrom > minFrom then
		self.tempFrom = minFrom
	end
	
	self.lastTableLen = ns:GetSortingTableLen()
	
--	ns.print('T', 'Scroll', self.tempFrom, ns:GetSortingTableLen())
	ns:UpdateScrollLines()
end)
		
local function SetArrowUp(self)
	local pi = math.pi
	local digree = 90
	local rads = digree*pi/180
	
	self:SetSize(18, 22)
	self:SetRotation(rads)
	self:Show()
	
	if self._OnShowHandler then
		self._OnShowHandler(self)
	end
end

local function SetArrowDown(self)
	local pi = math.pi
	local digree = 270
	local rads = digree*pi/180
	
	self:SetSize(18, 22)
	self:SetRotation(rads)
	self:Show()
	
	if self._OnShowHandler then
		self._OnShowHandler(self)
	end
end

local function SetArrowDisabled(self)
	self:SetSize(18, 0)
	self:Hide()
	
	if self._OnHideHandler then
		self._OnHideHandler(self)
	end
end

local function SetOnShowScript(self, func)

	self._OnShowHandler = func
end

local function SetOnHideScript(self, func)

	self._OnHideHandler = func
end

MainFrame.scrollFrame.bg = MainFrame.scrollFrame:CreateTexture(nil, 'BACKGROUND')
MainFrame.scrollFrame.bg:SetAllPoints()
ns.CustomSetTexture(MainFrame.scrollFrame.bg, 0.2,0.2,0.2,0.8)

MainFrame.scrollFrame.left = MainFrame.scrollFrame:CreateTexture(nil, 'OVERLAY')
MainFrame.scrollFrame.left:SetPoint('TOPLEFT', MainFrame.scrollFrame, 'TOPLEFT', 0, 0)
MainFrame.scrollFrame.left:SetPoint('BOTTOMLEFT', MainFrame.scrollFrame, 'BOTTOMLEFT', 0, 0)
MainFrame.scrollFrame.left:SetSize(1,1)
ns.CustomSetTexture(MainFrame.scrollFrame.left, 0,0,0,1)

MainFrame.scrollFrame.right = MainFrame.scrollFrame:CreateTexture(nil, 'OVERLAY')
MainFrame.scrollFrame.right:SetPoint('TOPRIGHT', MainFrame.scrollFrame, 'TOPRIGHT', 0, 0)
MainFrame.scrollFrame.right:SetPoint('BOTTOMRIGHT', MainFrame.scrollFrame, 'BOTTOMRIGHT', 0, 0)
MainFrame.scrollFrame.right:SetSize(1,1)
ns.CustomSetTexture(MainFrame.scrollFrame.right, 0,0,0,1)

MainFrame.scrollFrame.bottom = MainFrame.scrollFrame:CreateTexture(nil, 'OVERLAY')
MainFrame.scrollFrame.bottom:SetPoint('BOTTOMLEFT', MainFrame.scrollFrame, 'BOTTOMLEFT', 0, 0)
MainFrame.scrollFrame.bottom:SetPoint('BOTTOMRIGHT', MainFrame.scrollFrame, 'BOTTOMRIGHT', 0, 0)
MainFrame.scrollFrame.bottom:SetSize(1,1)
ns.CustomSetTexture(MainFrame.scrollFrame.bottom, 0,0,0,1)

MainFrame.scrollFrame.indexTab = CreateFrame('Frame', nil, MainFrame.scrollFrame)
MainFrame.scrollFrame.indexTab:SetSize(19, 20)
MainFrame.scrollFrame.indexTab.bg = MainFrame.scrollFrame.indexTab:CreateTexture(nil, 'BACKGROUND')
MainFrame.scrollFrame.indexTab.bg:SetAllPoints()
ns.CustomSetTexture(MainFrame.scrollFrame.indexTab.bg, 200/255, 199/255, 170/255,0.4)
MainFrame.scrollFrame.indexTab:SetPoint('TOPLEFT', MainFrame.scrollFrame, 'TOPLEFT', 1, -1)
	
MainFrame.scrollFrame.indexTab.text = MainFrame.scrollFrame.indexTab:CreateFontString(nil, 'ARTWORK')
MainFrame.scrollFrame.indexTab.text:SetFont(ns.default_font, ns.default_font_size, 'NONE')
MainFrame.scrollFrame.indexTab.text:SetShadowColor(0,0,0,1)
MainFrame.scrollFrame.indexTab.text:SetShadowOffset(1,-1)
MainFrame.scrollFrame.indexTab.text:SetText('№')
MainFrame.scrollFrame.indexTab.text:SetSize(1, 1)
MainFrame.scrollFrame.indexTab.text:SetAllPoints()

MainFrame.scrollFrame.specTab = CreateFrame('Frame', nil, MainFrame.scrollFrame)
MainFrame.scrollFrame.specTab:SetSize(30, 20)
MainFrame.scrollFrame.specTab.bg = MainFrame.scrollFrame.specTab:CreateTexture(nil, 'BACKGROUND')
MainFrame.scrollFrame.specTab.bg:SetAllPoints()
ns.CustomSetTexture(MainFrame.scrollFrame.specTab.bg, 170/255, 171/255, 200/255, 0.4)
MainFrame.scrollFrame.specTab:SetPoint('LEFT', MainFrame.scrollFrame.indexTab, 'RIGHT', 0, 0)

MainFrame.scrollFrame.nameTab = CreateFrame('Button', nil, MainFrame.scrollFrame)
MainFrame.scrollFrame.nameTab:SetSize(80, 20)
MainFrame.scrollFrame.nameTab.bg = MainFrame.scrollFrame.nameTab:CreateTexture(nil, 'BACKGROUND')
MainFrame.scrollFrame.nameTab.bg:SetAllPoints()
ns.CustomSetTexture(MainFrame.scrollFrame.nameTab.bg, 200/255, 199/255, 170/255,0.4)
sortingButtons[#sortingButtons+1] = MainFrame.scrollFrame.nameTab

MainFrame.scrollFrame.nameTab.highlight = MainFrame.scrollFrame.nameTab:CreateTexture(nil, 'OVERLAY')
MainFrame.scrollFrame.nameTab.highlight:SetAllPoints()
ns.CustomSetTexture(MainFrame.scrollFrame.nameTab.highlight, 1,1,1,0.4)
MainFrame.scrollFrame.nameTab.highlight:Hide()

MainFrame.scrollFrame.nameTab:SetPoint('LEFT', MainFrame.scrollFrame.specTab, 'RIGHT', 0, 0)
MainFrame.scrollFrame.nameTab.sorting = 1
MainFrame.scrollFrame.nameTab.revsorting = 5
MainFrame.scrollFrame.nameTab:SetScript('OnClick', ChangeSorting)
MainFrame.scrollFrame.nameTab:SetScript('OnEnter', function(self)
	self.highlight:Show()
end)
MainFrame.scrollFrame.nameTab:SetScript('OnLeave', function(self)
	self.highlight:Hide()
end)

MainFrame.scrollFrame.nameTab.text = MainFrame.scrollFrame.nameTab:CreateFontString(nil, 'ARTWORK')
MainFrame.scrollFrame.nameTab.text:SetFont(ns.default_font, ns.default_font_size, 'NONE')
MainFrame.scrollFrame.nameTab.text:SetShadowColor(0,0,0,1)
MainFrame.scrollFrame.nameTab.text:SetShadowOffset(1,-1)
MainFrame.scrollFrame.nameTab.text:SetText(L['GUI_PLAYER'])
MainFrame.scrollFrame.nameTab.text:SetSize(1, 1)
MainFrame.scrollFrame.nameTab.text:SetAllPoints()

MainFrame.scrollFrame.nameTab.arrow = MainFrame.scrollFrame.nameTab:CreateTexture(nil, 'ARTWORK')
MainFrame.scrollFrame.nameTab.arrow:SetSize(12, 16)
MainFrame.scrollFrame.nameTab.arrow:SetTexture("Interface\\ChatFrame\\ChatFrameExpandArrow")
MainFrame.scrollFrame.nameTab.arrow:SetPoint('LEFT', MainFrame.scrollFrame.nameTab, 'LEFT', 3, 0)
MainFrame.scrollFrame.nameTab.arrow.SetArrowUp = SetArrowUp
MainFrame.scrollFrame.nameTab.arrow.SetArrowDown = SetArrowDown
MainFrame.scrollFrame.nameTab.arrow.SetArrowDisabled = SetArrowDisabled
MainFrame.scrollFrame.nameTab.arrow.SetOnShowScript = SetOnShowScript
MainFrame.scrollFrame.nameTab.arrow.SetOnHideScript = SetOnHideScript
MainFrame.scrollFrame.nameTab.arrow:SetArrowUp()


MainFrame.scrollFrame.rollTypeTab = CreateFrame('Button', nil, MainFrame.scrollFrame)
MainFrame.scrollFrame.rollTypeTab:SetSize(80, 20)
MainFrame.scrollFrame.rollTypeTab.bg = MainFrame.scrollFrame.rollTypeTab:CreateTexture(nil, 'BACKGROUND')
MainFrame.scrollFrame.rollTypeTab.bg:SetAllPoints()
ns.CustomSetTexture(MainFrame.scrollFrame.rollTypeTab.bg, 170/255, 171/255, 200/255, 0.4)
MainFrame.scrollFrame.rollTypeTab:SetPoint('LEFT', MainFrame.scrollFrame.nameTab, 'RIGHT', 0, 0)
--MainFrame.scrollFrame.rollTypeTab.sorting = 2
--MainFrame.scrollFrame.rollTypeTab.revsorting = 6
--MainFrame.scrollFrame.rollTypeTab:SetScript('OnClick', ChangeSorting)
--[==[MainFrame.scrollFrame.rollTypeTab:SetScript('OnEnter', function(self)
	self.bg:SetTexture(200/255, 199/255, 170/255,0.8)
end)
MainFrame.scrollFrame.rollTypeTab:SetScript('OnLeave', function(self)
	self.bg:SetTexture(200/255, 199/255, 170/255,0.4)
end)
]==]

MainFrame.scrollFrame.rollTypeTab.text = MainFrame.scrollFrame.rollTypeTab:CreateFontString(nil, 'ARTWORK')
MainFrame.scrollFrame.rollTypeTab.text:SetFont(ns.default_font, ns.default_font_size, 'NONE')
MainFrame.scrollFrame.rollTypeTab.text:SetShadowColor(0,0,0,1)
MainFrame.scrollFrame.rollTypeTab.text:SetShadowOffset(1,-1)
MainFrame.scrollFrame.rollTypeTab.text:SetText(L['GUI_SELECT'])
MainFrame.scrollFrame.rollTypeTab.text:SetSize(1, 1)
MainFrame.scrollFrame.rollTypeTab.text:SetAllPoints()

MainFrame.scrollFrame.rangTab = CreateFrame('Frame', nil, MainFrame.scrollFrame)
MainFrame.scrollFrame.rangTab:SetSize(80, 20)
MainFrame.scrollFrame.rangTab.bg = MainFrame.scrollFrame.rangTab:CreateTexture(nil, 'BACKGROUND')
MainFrame.scrollFrame.rangTab.bg:SetAllPoints()
ns.CustomSetTexture(MainFrame.scrollFrame.rangTab.bg, 200/255, 199/255, 170/255,0.4)
MainFrame.scrollFrame.rangTab:SetPoint('LEFT', MainFrame.scrollFrame.rollTypeTab, 'RIGHT', 0, 0)

MainFrame.scrollFrame.rangTab.text = MainFrame.scrollFrame.rangTab:CreateFontString(nil, 'ARTWORK')
MainFrame.scrollFrame.rangTab.text:SetFont(ns.default_font, ns.default_font_size, 'NONE')
MainFrame.scrollFrame.rangTab.text:SetShadowColor(0,0,0,1)
MainFrame.scrollFrame.rangTab.text:SetShadowOffset(1,-1)
MainFrame.scrollFrame.rangTab.text:SetText(L['GUI_RANK'])
MainFrame.scrollFrame.rangTab.text:SetSize(1, 1)
MainFrame.scrollFrame.rangTab.text:SetAllPoints()

MainFrame.scrollFrame.rollNumberTab = CreateFrame('Button', nil, MainFrame.scrollFrame)
MainFrame.scrollFrame.rollNumberTab:SetSize(30, 20)
MainFrame.scrollFrame.rollNumberTab.bg = MainFrame.scrollFrame.rollNumberTab:CreateTexture(nil, 'BACKGROUND')
MainFrame.scrollFrame.rollNumberTab.bg:SetAllPoints()
ns.CustomSetTexture(MainFrame.scrollFrame.rollNumberTab.bg, 170/255, 171/255, 200/255, 0.4)
sortingButtons[#sortingButtons+1] = MainFrame.scrollFrame.rollNumberTab

MainFrame.scrollFrame.rollNumberTab.highlight = MainFrame.scrollFrame.rollNumberTab:CreateTexture(nil, 'OVERLAY')
MainFrame.scrollFrame.rollNumberTab.highlight:SetAllPoints()
ns.CustomSetTexture(MainFrame.scrollFrame.rollNumberTab.highlight, 1,1,1,0.4)
MainFrame.scrollFrame.rollNumberTab.highlight:Hide()

MainFrame.scrollFrame.rollNumberTab:SetPoint('LEFT', MainFrame.scrollFrame.rangTab, 'RIGHT', 0, 0)
MainFrame.scrollFrame.rollNumberTab.sorting = 3
MainFrame.scrollFrame.rollNumberTab.revsorting = 7
MainFrame.scrollFrame.rollNumberTab:SetScript('OnClick', ChangeSorting)
MainFrame.scrollFrame.rollNumberTab:SetScript('OnEnter', function(self)
	self.highlight:Show()
end)
MainFrame.scrollFrame.rollNumberTab:SetScript('OnLeave', function(self)
	self.highlight:Hide()
end)

MainFrame.scrollFrame.rollNumberTab.text = MainFrame.scrollFrame.rollNumberTab:CreateFontString(nil, 'ARTWORK')
MainFrame.scrollFrame.rollNumberTab.text:SetFont(ns.default_font, ns.default_font_size, 'NONE')
MainFrame.scrollFrame.rollNumberTab.text:SetShadowColor(0,0,0,1)
MainFrame.scrollFrame.rollNumberTab.text:SetShadowOffset(1,-1)
MainFrame.scrollFrame.rollNumberTab.text:SetText(L['GUI_DICEROLL'])
MainFrame.scrollFrame.rollNumberTab.text:SetSize(1, 1)
MainFrame.scrollFrame.rollNumberTab.text:SetAllPoints()
MainFrame.scrollFrame.rollNumberTab.text:Hide()

MainFrame.scrollFrame.rollNumberTab.texture = MainFrame.scrollFrame.rollNumberTab:CreateTexture(nil, 'ARTWORK')
MainFrame.scrollFrame.rollNumberTab.texture:SetDrawLayer('ARTWORK', 1)
MainFrame.scrollFrame.rollNumberTab.texture:SetTexture("Interface\\Buttons\\UI-GroupLoot-Dice-Up") -- L['GUI_DICEROLL'])
MainFrame.scrollFrame.rollNumberTab.texture:SetSize(14, 16)
MainFrame.scrollFrame.rollNumberTab.texture:SetPoint('CENTER', MainFrame.scrollFrame.rollNumberTab, 'CENTER', 0, 0)

MainFrame.scrollFrame.rollNumberTab.arrow = MainFrame.scrollFrame.rollNumberTab:CreateTexture(nil, 'ARTWORK')
MainFrame.scrollFrame.rollNumberTab.arrow:SetSize(12, 16)
MainFrame.scrollFrame.rollNumberTab.arrow:SetTexture("Interface\\ChatFrame\\ChatFrameExpandArrow")
MainFrame.scrollFrame.rollNumberTab.arrow:SetPoint('LEFT', MainFrame.scrollFrame.rollNumberTab, 'LEFT', -2, 0)
MainFrame.scrollFrame.rollNumberTab.arrow.SetArrowUp = SetArrowUp
MainFrame.scrollFrame.rollNumberTab.arrow.SetArrowDown = SetArrowDown
MainFrame.scrollFrame.rollNumberTab.arrow.SetArrowDisabled = SetArrowDisabled
MainFrame.scrollFrame.rollNumberTab.arrow.SetOnShowScript = SetOnShowScript
MainFrame.scrollFrame.rollNumberTab.arrow.SetOnHideScript = SetOnHideScript
MainFrame.scrollFrame.rollNumberTab.arrow:SetOnShowScript(function(self)
	MainFrame.scrollFrame.rollNumberTab.texture:ClearAllPoints()
	MainFrame.scrollFrame.rollNumberTab.texture:SetPoint('CENTER', MainFrame.scrollFrame.rollNumberTab, 'CENTER', 5, 0)
end)
MainFrame.scrollFrame.rollNumberTab.arrow:SetOnHideScript(function(self)
	MainFrame.scrollFrame.rollNumberTab.texture:ClearAllPoints()
	MainFrame.scrollFrame.rollNumberTab.texture:SetPoint('CENTER', MainFrame.scrollFrame.rollNumberTab, 'CENTER', 0, 0)
end)
MainFrame.scrollFrame.rollNumberTab.arrow:SetArrowUp()

MainFrame.scrollFrame.itemsTab = CreateFrame('Frame', nil, MainFrame.scrollFrame)
MainFrame.scrollFrame.itemsTab:SetSize(80, 20)
MainFrame.scrollFrame.itemsTab.bg = MainFrame.scrollFrame.itemsTab:CreateTexture(nil, 'BACKGROUND')
MainFrame.scrollFrame.itemsTab.bg:SetAllPoints()
ns.CustomSetTexture(MainFrame.scrollFrame.itemsTab.bg, 200/255, 199/255, 170/255,0.4)
MainFrame.scrollFrame.itemsTab:SetPoint('LEFT', MainFrame.scrollFrame.rollNumberTab, 'RIGHT', 0, 0)

MainFrame.scrollFrame.itemsTab.text = MainFrame.scrollFrame.itemsTab:CreateFontString(nil, 'ARTWORK')
MainFrame.scrollFrame.itemsTab.text:SetFont(ns.default_font, ns.default_font_size-2, 'NONE')
MainFrame.scrollFrame.itemsTab.text:SetShadowColor(0,0,0,1)
MainFrame.scrollFrame.itemsTab.text:SetShadowOffset(1,-1)
MainFrame.scrollFrame.itemsTab.text:SetText(L['GUI_EQPPED'])
MainFrame.scrollFrame.itemsTab.text:SetSize(1, 1)
MainFrame.scrollFrame.itemsTab.text:SetAllPoints()

MainFrame.scrollFrame.itemLevelTab = CreateFrame('Button', nil, MainFrame.scrollFrame)
MainFrame.scrollFrame.itemLevelTab:SetSize(40, 20)
MainFrame.scrollFrame.itemLevelTab.sorting = 4
MainFrame.scrollFrame.itemLevelTab.revsorting = 8
MainFrame.scrollFrame.itemLevelTab.bg = MainFrame.scrollFrame.itemLevelTab:CreateTexture(nil, 'BACKGROUND')
MainFrame.scrollFrame.itemLevelTab.bg:SetAllPoints()
ns.CustomSetTexture(MainFrame.scrollFrame.itemLevelTab.bg, 170/255, 171/255, 200/255, 0.4)
sortingButtons[#sortingButtons+1] = MainFrame.scrollFrame.itemLevelTab

MainFrame.scrollFrame.itemLevelTab.highlight = MainFrame.scrollFrame.itemLevelTab:CreateTexture(nil, 'OVERLAY')
MainFrame.scrollFrame.itemLevelTab.highlight:SetAllPoints()
ns.CustomSetTexture(MainFrame.scrollFrame.itemLevelTab.highlight, 1,1,1,0.4)
MainFrame.scrollFrame.itemLevelTab.highlight:Hide()

MainFrame.scrollFrame.itemLevelTab:SetPoint('LEFT', MainFrame.scrollFrame.itemsTab, 'RIGHT', 0, 0)
MainFrame.scrollFrame.itemLevelTab:SetScript('OnClick', ChangeSorting)
MainFrame.scrollFrame.itemLevelTab:SetScript('OnEnter', function(self)
	self.highlight:Show()
end)
MainFrame.scrollFrame.itemLevelTab:SetScript('OnLeave', function(self)
	self.highlight:Hide()
end)

MainFrame.scrollFrame.itemLevelTab.text = MainFrame.scrollFrame.itemLevelTab:CreateFontString(nil, 'ARTWORK')
MainFrame.scrollFrame.itemLevelTab.text:SetFont(ns.default_font, ns.default_font_size, 'NONE')
MainFrame.scrollFrame.itemLevelTab.text:SetShadowColor(0,0,0,1)
MainFrame.scrollFrame.itemLevelTab.text:SetShadowOffset(1,-1)
MainFrame.scrollFrame.itemLevelTab.text:SetText('iLvL')
MainFrame.scrollFrame.itemLevelTab.text:SetSize(30, 20)
MainFrame.scrollFrame.itemLevelTab.text:SetPoint('CENTER', MainFrame.scrollFrame.itemLevelTab, 'CENTER', 0, 0)

MainFrame.scrollFrame.itemLevelTab.arrow = MainFrame.scrollFrame.itemLevelTab:CreateTexture(nil, 'ARTWORK')
MainFrame.scrollFrame.itemLevelTab.arrow:SetSize(12, 16)
MainFrame.scrollFrame.itemLevelTab.arrow:SetTexture("Interface\\ChatFrame\\ChatFrameExpandArrow")
MainFrame.scrollFrame.itemLevelTab.arrow:SetPoint('LEFT', MainFrame.scrollFrame.itemLevelTab, 'LEFT', -2, 0)
MainFrame.scrollFrame.itemLevelTab.arrow.SetArrowUp = SetArrowUp
MainFrame.scrollFrame.itemLevelTab.arrow.SetArrowDown = SetArrowDown
MainFrame.scrollFrame.itemLevelTab.arrow.SetArrowDisabled = SetArrowDisabled
MainFrame.scrollFrame.itemLevelTab.arrow.SetOnShowScript = SetOnShowScript
MainFrame.scrollFrame.itemLevelTab.arrow.SetOnHideScript = SetOnHideScript
MainFrame.scrollFrame.itemLevelTab.arrow:SetOnShowScript(function(self)
	MainFrame.scrollFrame.itemLevelTab.text:ClearAllPoints()
	MainFrame.scrollFrame.itemLevelTab.text:SetPoint('CENTER', MainFrame.scrollFrame.itemLevelTab, 'CENTER', 5, 0)
end)
MainFrame.scrollFrame.itemLevelTab.arrow:SetOnHideScript(function(self)
	MainFrame.scrollFrame.itemLevelTab.text:ClearAllPoints()
	MainFrame.scrollFrame.itemLevelTab.text:SetPoint('CENTER', MainFrame.scrollFrame.itemLevelTab, 'CENTER', 0, 0)
end)
MainFrame.scrollFrame.itemLevelTab.arrow:SetArrowUp()

MainFrame.scrollFrame.noteTab = CreateFrame('Button', nil, MainFrame.scrollFrame)
MainFrame.scrollFrame.noteTab:SetSize(60, 20)
MainFrame.scrollFrame.noteTab.bg = MainFrame.scrollFrame.noteTab:CreateTexture(nil, 'BACKGROUND')
MainFrame.scrollFrame.noteTab.bg:SetAllPoints()
ns.CustomSetTexture(MainFrame.scrollFrame.noteTab.bg, 200/255, 199/255, 170/255,0.4)
MainFrame.scrollFrame.noteTab:SetPoint('LEFT', MainFrame.scrollFrame.itemLevelTab, 'RIGHT', 0, 0)
MainFrame.scrollFrame.noteTab:SetScript('OnClick', ChangeSorting)

MainFrame.scrollFrame.noteTab.text = MainFrame.scrollFrame.noteTab:CreateFontString(nil, 'ARTWORK')
MainFrame.scrollFrame.noteTab.text:SetFont(ns.default_font, ns.default_font_size, 'NONE')
MainFrame.scrollFrame.noteTab.text:SetShadowColor(0,0,0,1)
MainFrame.scrollFrame.noteTab.text:SetShadowOffset(1,-1)
MainFrame.scrollFrame.noteTab.text:SetText(L['GUI_NOTES'])
MainFrame.scrollFrame.noteTab.text:SetSize(1, 1)
MainFrame.scrollFrame.noteTab.text:SetAllPoints()


MainFrame.selectTab = CreateFrame('Frame', nil, MainFrame)
MainFrame.selectTab:SetSize(38, 38)
MainFrame.selectTab:SetPoint('BOTTOM', MainFrame.scrollFrame, 'TOP', 0, 0)

MainFrame.selectTab.bg = MainFrame.selectTab:CreateTexture()
MainFrame.selectTab.bg:SetAllPoints()
ns.CustomSetTexture(MainFrame.selectTab.bg, 0.1,0.1,0.1,0.8)
MainFrame.selectTab.bg:SetDrawLayer('ARTWORK', -1)

MainFrame.selectTab.left = MainFrame.selectTab:CreateTexture()
MainFrame.selectTab.left:SetPoint('TOPLEFT', MainFrame.selectTab, 'TOPLEFT', 0, 0)
MainFrame.selectTab.left:SetPoint('BOTTOMLEFT', MainFrame.selectTab, 'BOTTOMLEFT', 0, 0)
MainFrame.selectTab.left:SetSize(1,1)
ns.CustomSetTexture(MainFrame.selectTab.left, 0,0,0,1)
MainFrame.selectTab.left:SetDrawLayer('ARTWORK', 1)

MainFrame.selectTab.right = MainFrame.selectTab:CreateTexture()
MainFrame.selectTab.right:SetPoint('TOPRIGHT', MainFrame.selectTab, 'TOPRIGHT', 0, 0)
MainFrame.selectTab.right:SetPoint('BOTTOMRIGHT', MainFrame.selectTab, 'BOTTOMRIGHT', 0, 0)
MainFrame.selectTab.right:SetSize(1,1)
ns.CustomSetTexture(MainFrame.selectTab.right, 0,0,0,1)
MainFrame.selectTab.right:SetDrawLayer('ARTWORK', 1)

MainFrame.selectTab.top = MainFrame.selectTab:CreateTexture()
MainFrame.selectTab.top:SetPoint('TOPLEFT', MainFrame.selectTab, 'TOPLEFT', 0, 0)
MainFrame.selectTab.top:SetPoint('TOPRIGHT', MainFrame.selectTab, 'TOPRIGHT', 0, 0)
MainFrame.selectTab.top:SetSize(1,1)
ns.CustomSetTexture(MainFrame.selectTab.top, 0,0,0,1)
MainFrame.selectTab.top:SetDrawLayer('ARTWORK', 1)

MainFrame.selectTab.bottomLeft = MainFrame.selectTab:CreateTexture()
MainFrame.selectTab.bottomLeft:SetPoint('TOPRIGHT', MainFrame.selectTab, 'BOTTOMLEFT', 0, 0)
MainFrame.selectTab.bottomLeft:SetPoint('TOPLEFT', MainFrame.scrollFrame, 'TOPLEFT', 0, 0)
MainFrame.selectTab.bottomLeft:SetSize(1,1)
ns.CustomSetTexture(MainFrame.selectTab.bottomLeft, 0,0,0,1)
MainFrame.selectTab.bottomLeft:SetDrawLayer('ARTWORK', 1)

MainFrame.selectTab.bottomRight = MainFrame.selectTab:CreateTexture()
MainFrame.selectTab.bottomRight:SetPoint('TOPLEFT', MainFrame.selectTab, 'BOTTOMRIGHT', 0, 0)
MainFrame.selectTab.bottomRight:SetPoint('TOPRIGHT', MainFrame.scrollFrame, 'TOPRIGHT', 0, 0)
MainFrame.selectTab.bottomRight:SetSize(1,1)
ns.CustomSetTexture(MainFrame.selectTab.bottomRight, 0,0,0,1)
MainFrame.selectTab.bottomRight:SetDrawLayer('ARTWORK', 1)



local function LineButton_GiveItOnEnter(self, button)
	self.highlight:Show()
end

local function LineButton_GiveItOnLeave(self, button)
	self.highlight:Hide()
end

do
	local animList = {}
	local anim = CreateFrame('Frame')
	anim:Hide()
	anim.elapsed = 0
	
	
	local function UpdateFrames(timer)
		for i=1, #animList do
			local d = animList[i]	
			
			local alpha = d.anim - timer/1
			
			if alpha > 0 then
				
				local a1 = min(1, alpha/0.7)
				d.warningbg:SetAlpha(a1)
				d.warning:SetAlpha(a1)
					
					local t = 1-alpha
				d.text:SetAlpha(t*t)

				
			--	print('T', alpha, d.anim - timer/1 )
			elseif d.anim <= timer then	
				tremove(animList, i)
				
				d.warningbg:SetAlpha(0)
				d.warning:SetAlpha(0)
				d.text:SetAlpha(1)
				
	--			print('SuccessGiveItButton 2', d, GetTime())
				
				UpdateFrames(timer)			
				break
			end
		end	
	end
	
	anim:SetScript('OnUpdate', function(self, elapsed)
		self.elapsed = (self.elapsed or 0) + elapsed		
		if #animList == 0 then self:Hide() end
		UpdateFrames(GetTime())
	end)
	
	function ns.ErrorGiveItButton(button, errorType)
		
		button.warningbg:SetVertexColor(0.9, 0.2, 0.2, 1)	
		button.onEnd = nil
		
		for i=1, #animList do
			if animList[i] == button then
				animList[i].anim = GetTime() + 1
				animList[i].errorType = errorType
				animList[i].warning:SetText(L['LOOT_ERROR_'..errorType])
				anim:Show()
				return
			end
		end
		
		local id = #animList+1
		
		animList[id] = button
		animList[id].anim = GetTime() + 1
		animList[id].errorType = errorType
		animList[id].warning:SetText(L['LOOT_ERROR_'..errorType])
		
		anim:Show()
		
		
	end

	function ns.SuccessGiveItButton(button)
		
		button.warningbg:SetVertexColor(0.2, 0.9, 0.2, 1)

		for i=1, #animList do
			if animList[i] == button then
				animList[i].anim = GetTime() + 1
				animList[i].errorType = -1
				animList[i].warning:SetText(L['ITEM_AWARDED'])
				anim:Show()
				return
			end
		end
		
		local id = #animList+1
		
		animList[id] = button
		animList[id].anim = GetTime() + 1
		animList[id].errorType = -1
		animList[id].warning:SetText(L['ITEM_AWARDED'])
		anim:Show()

	end
	
	function ns.BreakGiveItButton(button)
	
		for i=1, #animList do
			if animList[i] == button then			
				tremove(animList, i)				
				animList[i].warningbg:SetAlpha(0)
				animList[i].warning:SetAlpha(0)
				animList[i].text:SetAlpha(1)				
				return
			end
		end
		
		if #animList == 0 then anim:Hide() end
	end
	
end

function ns.ItemIsAwarded_SendCloseSession(rollID, button, line, selectedName)

	if selectedName == MainFrame.scrollFrame.selectedName then
		MainFrame.scrollFrame.selectedName = nil
		ns.CustomSetTexture(line.bg, 0.8,0.8,0.8,0.2)
		button:Hide()
		ns:UpdateColoredFilterSettings()
	end
	
	ns:SendMessage(rollID, ns.channelNameLootRollCancel)
	
	local guid, index, link = strsplit('\n', rollID)

	ns.Server_RemoveItemGroup(link)
	
--	print('T', 'ItemIsAwarded_SendCloseSession', rollID)
end

function ns.ItemIsAwarded_UnselectPlayer( button, line, selectedName )

	if selectedName == MainFrame.scrollFrame.selectedName then
		MainFrame.scrollFrame.selectedName = nil
		ns.CustomSetTexture(line.bg, 0.8,0.8,0.8,0.2)
		button:Hide()
		ns:UpdateColoredFilterSettings()
	end
	
--	print('T', 'ItemIsAwarded_UnselectPlayer', selectedName)
end

function ns.ItemIsAwarded(buttonType, slot, link, name, syncMsg, button, line)
	
--	print('LH', ns.ShortLinkName(link), '->', ns.GetColoredName(name), '(|cFF00FF00'..L['ITEM_AWARDED']..'|r)', 'BT:',buttonType)
		
	ns:SendMessage( syncMsg , ns.channelNameLootRollAward)
	
--	print('SuccessGiveItButton', button, GetTime())
	ns.SuccessGiveItButton(button)
	
	if ns.Server_ItemIsGrouped(link) then
		ns.UpdateIconGrid_NumItems(line._rollID)
		if ns.Server_GetAmountGroupedItems(link) == 0 then 	
--			print('T', 'Server_GetAmountGroupedItems', ns.Server_GetAmountGroupedItems(link), link)
			ns.QueueFunction(0.7, 'ItemIsAwarded_SendCloseSession', line._rollID, button, line, MainFrame.scrollFrame.selectedName)
		else
			ns.QueueFunction(0.7, 'ItemIsAwarded_UnselectPlayer', button, line, MainFrame.scrollFrame.selectedName)
		end
	else
		ns.QueueFunction(1, 'ItemIsAwarded_SendCloseSession', line._rollID, button, line, MainFrame.scrollFrame.selectedName)
	end	
	--[==[
	if ns.Server_ItemIsGrouped(link) then
		if ns.Server_GetAmountGroupedItems(link) > 0 then 
			ns.SuccessGiveItButton(button)
		end
	else
		MainFrame.scrollFrame.selectedName = nil
		line.bg:SetTexture(0.8,0.8,0.8,0.2)
		button:Hide()
		ns:SendMessage(line._rollID, ns.channelNameLootRollCancel)
	end
	]==]
	if ns.Slot_QueueList[slot] then
		ns.BreakQueueFunctionByID(ns.Slot_QueueList[slot])		
		ns.Slot_QueueList[slot] = nil
	end
end

local getFullError = false
local lastName = nil
local lastInvFullTime = 0
local lastIndexQueue = nil

function ns.WaitForAwarding(buttonType, slot, link, name, syncMsg, button, line)

	local link_award_check = GetLootSlotLink(slot)
		
	if link_award_check then
		if getFullError and ( name == lastName ) then 
			ns.ErrorGiveItButton(button, 5)
			getFullError = false
			lastIndexQueue = nil
		else
			ns.ErrorGiveItButton(button, 3)
		end
	
		if ns.Slot_QueueList[slot] then
			ns.Slot_QueueList[slot] = false
		end
	else
		
		ns.ItemIsAwarded(buttonType, slot, link, name, syncMsg, button, line)
	end
end

local function LineButton_GiveItOnClick(self, button)
	if button ~= 'RightButton' then return end
	
	if self.skipTimeOut and self.skipTimeOut > GetTime() then
		return
	end
	
	local line = self:GetParent()
	
	ns.print('T', 'LineButton_GiveItOnClick', button)
	
	local slot = ns:GetMasterLootItemSlotByLink(line._link)
	
	ns.print('T', 'Slot', slot)
	
	if not slot then 		
		ns.ErrorGiveItButton(self, 1)
		return 
	end
	
	if ns:IsMasterLooter(true) then
		local index = ns:GetMasterLootCandidateIndex(slot, line._name)

		ns.print('T', 'Index', index)
		
		if not index then		
			ns.ErrorGiveItButton(self, 2)
			return 
		end
		
		ns.print('T', 'GiveMasterLoot', slot, index)
		
		local name, className, classFile = GetMasterLootCandidate(slot, index)

		local link = GetLootSlotLink(slot)
		local untiName, unitServer = ns:GetUnitFullName(name)
		
		if not untiName then	
			ns.ErrorGiveItButton(self, 4)
			return 
		end
		
		lastName = name
		lastInvFullTime = GetTime()
		GiveMasterLoot(slot, index, 'LHRun')
		
		self.skipTimeOut = GetTime() + 1
		local bossID = ns:GetItemBossInfo(link) or 0
		
		local success, amountIndex
		local syncMsg = untiName..'-'..unitServer..'\n'..classFile..'\n'..link..'\n'..(line._buttonID or 0)..'\n'..(line._randIndex or -1)..'\n'..bossID
		if not ns.Slot_QueueList[slot] then
			amountIndex = ns.QueueFunction(ns.timeWaitForCheck, 'WaitForAwarding', 'AddOn', slot, link, name, syncMsg, self, line)
			
			ns.Slot_QueueList[slot] = amountIndex
		else
			success, amountIndex = ns.UpdateQueueFunction(ns.Slot_QueueList[slot], ns.timeWaitForCheck, 'WaitForAwarding', 'AddOn', slot, link, name, syncMsg, self, line)
			
		end
		
		lastIndexQueue = amountIndex
	--	print('T', amountIndex, success)
	else
	
		ns.print('T', 'LootSlot', slot)
		LootSlot(slot)
		
		MainFrame.scrollFrame.selectedName = nil
		ns.CustomSetTexture(line.bg, 0.8,0.8,0.8,0.2)
		self:Hide()
	end
end

function events:LOOT_SLOT_CLEARED(event, slotID)	
	ns.dprint(event, slotID)	
	if ns.Slot_QueueList[slotID] then	
		local buttonType, slot, link, name, syncMsg, button, line = ns.GetQueueFunctionArgs(ns.Slot_QueueList[slotID])	
		
		ns.dprint('Slot_Cleared', buttonType, slot, link, name, syncMsg, button, line)
		
		if buttonType == 'AddOn' then
			ns.ItemIsAwarded(buttonType, slot, link, name, syncMsg, button, line)
		elseif buttonType == 'Blizzard' then
			ns.ItemIsAwarded_Blizzard(buttonType, slot, link, name, syncMsg)
		end
	elseif ns.Slot_QueueList[slotID] == false then
	
		ns.dprint('Slot_Cleared', slotID, 'Items is awarded but queue is closed')
	end
end

function events:LOOT_OPENED(event)
	wipe(ns.Slot_QueueList)

end

function events:UI_ERROR_MESSAGE(event, message)
	if message == _G['ERR_INV_FULL'] then
		getFullError = true
	--	print('T', GetTime() - lastInvFullTime)
		if lastIndexQueue then
			local modif, slot, link, name, syncMsg, me, line = ns.GetQueueFunctionArgs(lastIndexQueue)
			local success, amountIndex = ns.UpdateQueueFunction(lastIndexQueue, 0.1, 'WaitForAwarding', modif, slot, link, name, syncMsg, me, line)
			
		end
	end
end

function events.LOOT_SLOT_CHANGED(event, slotID)
	
	ns.dprint(event, slotID)
end

function events:LOOT_CLOSED(event)
	ns.BreakQueueFunctionByName('WaitForAwarding')
	ns.BreakQueueFunctionByName('WaitForAwarding_Blizzard')
	
	ns.dprint('Loot Closed')
--	wipe(ns.Slot_QueueList)	
end

local function LineButton_OnEnter(self)

	local r, g, b, rs, gs, bs = ns:UnpackFilterIndexColor(self.filterIndex, self._rollType)
	
	if MainFrame.scrollFrame.selectedName == self._name then
		ns.CustomSetTexture(self.bg, rs, gs, bs, 0.4)
	else
		ns.CustomSetTexture(self.bg, r, g, b,0.4)
	end
end

local function LineButton_OnLeave(self)
	local r, g, b, rs, gs, bs = ns:UnpackFilterIndexColor(self.filterIndex, self._rollType)
	
	if MainFrame.scrollFrame.selectedName == self._name then
		ns.CustomSetTexture(self.bg, rs, gs, bs, 0.4)
	else
		ns.CustomSetTexture(self.bg, r, g, b,0.2)
	end
end

local function LineButton_OnClick(self)
	local r, g, b, rs, gs, bs = ns:UnpackFilterIndexColor(self.filterIndex, self._rollType)
	MainFrame.scrollFrame.selectedName = self._name
	ns.CustomSetTexture(self.bg, rs, gs, bs, 0.4)
	
	ns.BreakGiveItButton(self.giveIt)
	
	ns:UpdateColoredFilterSettings()
end

local rollTypeOverlay_OnEnter = function(self)
	GameTooltip:SetOwner(self, "ANCHOR_RIGHT")
	GameTooltip:SetText('Правая кнопка мыши - Изменить выбор')
end

local rollTypeOverlay_OnLeave = function(self)
	GameTooltip:Hide()
end

local rollTypeOverlay_OnClick = function(self, button)
	ns.ShowMenu(self, ns.BuildRollTypeMenuList(self))
end

-- local buttonName, buttonTip, buttonArg = ns:GetButtonIDInfo(tonumber(data.button))
-- local color = rollTypeColors[buttonTip] or ''
function ns.BuildRollTypeMenuList(overlay)
	local t = {}
	local list = ns.GetButtonInfoList()
	local color
	local buttonID = overlay:GetParent()._buttonID
	
	for i=1, #list do	
		color = rollTypeColors[list[i].tip] or ''		
		
		if buttonID == i then
		
		else
			t[#t+1] = { text = color..list[i].name, notCheckable = true, func = function(self) 			
				ns:UpdateRollType_OnSend(overlay:GetParent()._rollID, i)
			end }
		end
		
	end
	
	local passName, passtip, passarg = ns:GetButtonIDInfo(0)
	color = rollTypeColors[passtip] or ''
	
	if buttonID == 0 then
	
	else
		t[#t+1] = { text = color..passName, notCheckable = true, func = function(self)	
			ns:UpdateRollType_OnSend(overlay:GetParent()._rollID, 0)
		end }
	end
	
	t[#t+1] = { text = '', disabled = true, notCheckable = true }
	t[#t+1] = { text = '|cFFcbcbcbОтмена|r', func = function(self) ns.HideMenu() end, notCheckable = true }
	
	return t
end

local function CreateInfoLine(frame)
	local line = CreateFrame('Button', nil, frame)
	line:SetScript('OnEnter', LineButton_OnEnter)
	line:SetScript('OnLeave', LineButton_OnLeave)
	line:SetScript('OnClick', LineButton_OnClick)

	line.bg = line:CreateTexture()
	line.bg:SetAllPoints()
	ns.CustomSetTexture(line.bg, 0.8,0.8,0.8,0.2)
	line.bg:SetDrawLayer('ARTWORK', 0)
	
	line.index = line:CreateFontString()
	line.index:SetFontObject('GameFontWhite')
	line.index:SetDrawLayer('ARTWORK', 1)
	line.index:SetFont(ns.default_font, ns.default_font_size, ns.default_font_outline)
	line.index:SetShadowColor(0,0,0,1)
	line.index:SetShadowOffset(1,-1)
	line.index:SetText('#1.')
	line.index:SetSize(20, 20)
	line.index:SetPoint('LEFT', line, 'LEFT', 0, 0)
	line.index:SetJustifyH("LEFT")
	
	line.classOrSpecIcon = line:CreateTexture()
	line.classOrSpecIcon:SetDrawLayer('ARTWORK', 1)
	line.classOrSpecIcon:SetSize(18,18)
	line.classOrSpecIcon:SetPoint('LEFT', line.index, 'RIGHT', 5, 0)
	
	line.name = line:CreateFontString()
	line.name:SetFontObject('GameFontWhite')
	line.name:SetDrawLayer('ARTWORK', 1)
	line.name:SetFont(ns.default_font, ns.default_font_size, ns.default_font_outline)
	line.name:SetText('< name >')
	line.name:SetSize(80, 20)
	line.name:SetPoint('LEFT', line.classOrSpecIcon, 'RIGHT', 5+2, 0)
	line.name:SetShadowColor(0,0,0,1)
	line.name:SetShadowOffset(1,-1)
	line.name:SetJustifyH("CENTER")
	
	line.rollType = line:CreateFontString()
	line.rollType:SetFontObject('GameFontWhite')
	line.rollType:SetDrawLayer('ARTWORK', 1)
	line.rollType:SetFont(ns.default_font, ns.default_font_size, ns.default_font_outline)
	line.rollType:SetText('< rollType >')
	line.rollType:SetSize(80, 20)
	line.rollType:SetPoint('LEFT', line.name, 'RIGHT', 0, 0)
	line.rollType:SetShadowColor(0,0,0,1)
	line.rollType:SetShadowOffset(1,-1)
	line.rollType:SetJustifyH("CENTER")
	
	line.rollType.glow = line:CreateTexture(nil, 'ARTWORK', nil, 0)
	line.rollType.glow:SetVertexColor(1,1,1,1)
	line.rollType.glow:SetBlendMode('ADD')
	line.rollType.glow:SetSize(80, 18)
	line.rollType.glow:SetPoint('BOTTOM', line.rollType, 'BOTTOM', 0, 0)
	line.rollType.glow:SetTexture('Interface\\ChatFrame\\ChatFrameTab-NewMessage')
	line.rollType.glow:SetAlpha(0)
	line.rollType.glow:Show()
	
	local ag = line.rollType.glow:CreateAnimationGroup()
	local a1 = ag:CreateAnimation("Alpha")
	if ns.IsLegion then
		a1:SetFromAlpha(0)
		a1:SetToAlpha(0.6)
		a1:SetDuration(0.3)
	else
		a1:SetChange(0.6)
		a1:SetDuration(0.3)
	end
	a1:SetOrder(1)
	local a2 = ag:CreateAnimation("Alpha")
	if ns.IsLegion then
		a2:SetFromAlpha(0.6)
		a2:SetToAlpha(0)
		a2:SetDuration(0.5)
	else
		a2:SetChange(-0.6)
		a2:SetDuration(0.5)
	end
	a2:SetOrder(2)
	
	line.rollType.glow.ag = ag
	
	line.Glow = function(self)
		line.rollType.glow.ag:Play()
	end
		
	line.rollTypeOverlay = CreateFrame("Button", nil, line)
	line.rollTypeOverlay:SetPoint("CENTER", line.rollType, 'CENTER', 0, 0)
	line.rollTypeOverlay:SetSize(80, 20)
	line.rollTypeOverlay:RegisterForClicks("RightButtonUp")
	line.rollTypeOverlay:SetScript("OnEnter", rollTypeOverlay_OnEnter)
	line.rollTypeOverlay:SetScript("OnLeave", rollTypeOverlay_OnLeave)
	line.rollTypeOverlay:SetScript('OnClick', rollTypeOverlay_OnClick)

	line.rollTypeOverlay.texture = line.rollTypeOverlay:CreateTexture()
	line.rollTypeOverlay.texture:SetDrawLayer('ARTWORK', 1)
	line.rollTypeOverlay.texture:SetPoint('TOPLEFT', line.rollTypeOverlay, 'TOPLEFT', 3, -3)
	line.rollTypeOverlay.texture:SetPoint('BOTTOMRIGHT', line.rollTypeOverlay, 'BOTTOMRIGHT', -3, 3)
	ns.CustomSetTexture(line.rollTypeOverlay.texture, 1, 1, 1, 0.1)
	
	line.rang = line:CreateFontString()
	line.rang:SetFontObject('GameFontWhite')
	line.rang:SetDrawLayer('ARTWORK', 1)
	line.rang:SetFont(ns.default_font, ns.default_font_size, ns.default_font_outline)
	line.rang:SetText('< rang >')
	line.rang:SetSize(80, 20)
	line.rang:SetPoint('LEFT', line.rollType, 'RIGHT', 0, 0)
	line.rang:SetShadowColor(0,0,0,1)
	line.rang:SetShadowOffset(1,-1)
	line.rang:SetJustifyH("CENTER")
	
	line.rollNumber = line:CreateFontString()
	line.rollNumber:SetFontObject('GameFontWhite')
	line.rollNumber:SetDrawLayer('ARTWORK', 1)
	line.rollNumber:SetFont(ns.default_font, ns.default_font_size, ns.default_font_outline)
	line.rollNumber:SetText('< rollNumber >')
	line.rollNumber:SetSize(30, 20)
	line.rollNumber:SetPoint('LEFT', line.rang, 'RIGHT', 0, 0)
	line.rollNumber:SetShadowColor(0,0,0,1)
	line.rollNumber:SetShadowOffset(1,-1)
	line.rollNumber:SetJustifyH("CENTER")
	
	line.items = line:CreateFontString()
	line.items:SetFontObject('GameFontWhite')
	line.items:SetDrawLayer('ARTWORK', 1)
	line.items:SetFont(ns.default_font, ns.default_font_size, ns.default_font_outline)
	line.items:SetText('')
	line.items:SetSize(80, 20)
	line.items:SetPoint('LEFT', line.rollNumber, 'RIGHT', 0, 0)
	line.items:SetShadowColor(0,0,0,1)
	line.items:SetShadowOffset(1,-1)
	
	line.items1 = CreateFrame("Button", nil, line)
	line.items1:SetPoint("LEFT", line.items, 'LEFT', 17, 0)
	line.items1:SetSize(14, 14)
	line.items1:SetScript("OnEnter", SetItemTip)
	line.items1:SetScript("OnLeave", HideTip2)
--	line.items1:SetScript("OnUpdate", ItemOnUpdate)
	line.items1:SetScript('OnEvent', SetItemTip)
	
	line.items1:SetScript("OnClick", LootClick)	
	line.items1.icon = line.items1:CreateTexture(nil, 'OVERLAY')
	line.items1.icon:SetAllPoints()
	line.items1.icon:SetTexCoord(0.07, 0.93, 0.07, 0.93)
	
	line.items2 = CreateFrame("Button", nil, line)
	line.items2:SetPoint("LEFT", line.items1, 'RIGHT', 2, 0)
	line.items2:SetSize(14, 14)
	line.items2:SetScript("OnEnter", SetItemTip)
	line.items2:SetScript("OnLeave", HideTip2)
--	line.items2:SetScript("OnUpdate", ItemOnUpdate)
	line.items2:SetScript('OnEvent', SetItemTip)
	line.items2:SetScript("OnClick", LootClick)	
	line.items2.icon = line.items2:CreateTexture(nil, 'OVERLAY')
	line.items2.icon:SetAllPoints()
	line.items2.icon:SetTexCoord(0.07, 0.93, 0.07, 0.93)
	
	line.items3 = CreateFrame("Button", nil, line)
	line.items3:SetPoint("LEFT", line.items2, 'RIGHT', 2, 0)
	line.items3:SetSize(14, 14)
	line.items3:SetScript("OnEnter", SetItemTip)
	line.items3:SetScript("OnLeave", HideTip2)
--	line.items3:SetScript("OnUpdate", ItemOnUpdate)
	line.items3:SetScript('OnEvent', SetItemTip)
	line.items3:SetScript("OnClick", LootClick)	
	line.items3.icon = line.items3:CreateTexture(nil, 'OVERLAY')
	line.items3.icon:SetAllPoints()
	line.items3.icon:SetTexCoord(0.07, 0.93, 0.07, 0.93)
	
	
	line.itemLvl = line:CreateFontString()
	line.itemLvl:SetFontObject('GameFontWhite')
	line.itemLvl:SetDrawLayer('ARTWORK', 1)
	line.itemLvl:SetFont(ns.default_font, ns.default_font_size, ns.default_font_outline)
	line.itemLvl:SetText('ItemLevel')
	line.itemLvl:SetSize(40, 20)
	line.itemLvl:SetPoint('LEFT', line.items, 'RIGHT', 0, 0)
	line.itemLvl:SetShadowColor(0,0,0,1)
	line.itemLvl:SetShadowOffset(1,-1)
	line.itemLvl:SetJustifyH("CENTER")
	
	line.note = line:CreateFontString()
	line.note:SetFontObject('GameFontWhite')
	line.note:SetDrawLayer('ARTWORK', 1)
	line.note:SetFont(ns.default_font, ns.default_font_size, ns.default_font_outline)
	line.note:SetText('')
	line.note:SetSize(60, 20)
	line.note:SetPoint('LEFT', line.itemLvl, 'RIGHT', 0, 0)
	line.note:SetShadowColor(0,0,0,1)
	line.note:SetShadowOffset(1,-1)
	
	line.noteOverlay = CreateFrame("Button", nil, line)
	line.noteOverlay:SetPoint("CENTER", line.note, 'CENTER', 0, 0)
	line.noteOverlay:SetSize(40, 20)
	line.noteOverlay:SetScript("OnEnter", function(self)
		GameTooltip:SetOwner(self, "ANCHOR_RIGHT")
		GameTooltip:SetText(self.text)
	end)
	line.noteOverlay:SetScript("OnLeave", function(self)
		GameTooltip:Hide()
	end)


	line.giveIt = CreateFrame('Button',nil, line)
	line.giveIt:RegisterForClicks("RightButtonUp")
	line.giveIt:SetSize(82, 16)	
	line.giveIt:SetPoint('LEFT', line.note, 'RIGHT', 3, 0)
	line.giveIt:SetScript('OnClick', LineButton_GiveItOnClick)
	line.giveIt:SetScript('OnEnter', LineButton_GiveItOnEnter)
	line.giveIt:SetScript('OnLeave', LineButton_GiveItOnLeave)
	line.giveIt:SetScript('OnMouseUp', function(self, button)
		if button ~= 'RightButton' then return end
		self.text:SetPoint('CENTER', self, 'CENTER', 0, 0)
	end)
	line.giveIt:SetScript('OnMouseDown', function(self, button)
		if button ~= 'RightButton' then return end
		self.text:SetPoint('CENTER', self, 'CENTER', 1, -1)
	end)
	
	line.giveIt.bg = line.giveIt:CreateTexture()
	line.giveIt.bg:SetDrawLayer('OVERLAY', 1)
	line.giveIt.bg:SetAllPoints()
	line.giveIt.bg:SetTexture(ns.buttonsTexture)
	line.giveIt.bg:SetVertexColor(0.6, 0.6, 0.1, 1)

	line.giveIt.highlight = line.giveIt:CreateTexture()
	line.giveIt.highlight:SetDrawLayer('OVERLAY', 2)
	line.giveIt.highlight:SetAllPoints()
	ns.CustomSetTexture(line.giveIt.highlight, 1,1,1,0.2)
	line.giveIt.highlight:Hide()

	line.giveIt.border = line.giveIt:CreateTexture(nil, 'OVERLAY', nil, 0)
	line.giveIt.border:SetPoint('TOPLEFT', line.giveIt, 'TOPLEFT', -1, 1)
	line.giveIt.border:SetPoint('BOTTOMRIGHT', line.giveIt, 'BOTTOMRIGHT', 1, -1)
	ns.CustomSetTexture(line.giveIt.border, 0, 0, 0, 1)
	
	line.giveIt.text = line.giveIt:CreateFontString(nil, 'OVERLAY', nil, 0)
	line.giveIt.text:SetPoint('CENTER', line.giveIt, 'CENTER', 0, 0)
	line.giveIt.text:SetFont(ns.default_font, ns.default_font_size+2, ns.default_font_outline)
	line.giveIt.text:SetText('Отдать')
	line.giveIt.text:SetShadowColor(0,0,0,1)
	line.giveIt.text:SetShadowOffset(1,-1)
	
	line.giveIt.warning = line.giveIt:CreateFontString(nil, 'OVERLAY', nil, 1)
	line.giveIt.warning:SetPoint('CENTER', line.giveIt, 'CENTER', 0, 0)
	line.giveIt.warning:SetFont(ns.default_font, ns.default_font_size+2, ns.default_font_outline)
	line.giveIt.warning:SetText('Error')
	line.giveIt.warning:SetShadowColor(0,0,0,1)
	line.giveIt.warning:SetShadowOffset(1,-1)
	line.giveIt.warning:SetAlpha(0)
	
	line.giveIt.warningbg = line.giveIt:CreateTexture(nil, 'OVERLAY', nil, 1)
	line.giveIt.warningbg:SetAllPoints()
	line.giveIt.warningbg:SetTexture(ns.buttonsTexture)
	line.giveIt.warningbg:SetVertexColor(0.9, 0.2, 0.2, 1)	
	line.giveIt.warningbg:SetAlpha(0)
	
	return line
end

for i=1, 10 do
	local height = floor( MainFrame.scrollFrame:GetHeight() / 10  ) - 3
	
	MainFrame.scrollFrame.lines[i] = CreateInfoLine(MainFrame.scrollFrame)
	MainFrame.scrollFrame.lines[i]:SetPoint('TOP', MainFrame.scrollFrame, 'TOP', 0 , -22 - ( ( height + 1 ) * (i-1) ))
	MainFrame.scrollFrame.lines[i]:SetWidth( MainFrame.scrollFrame:GetWidth() - 2)
	MainFrame.scrollFrame.lines[i]:SetHeight( height )
	
end

function ns:ShowGiveItButtons()
	if MainFrame.scrollFrame.selectedName then
		for i=1, #MainFrame.scrollFrame.lines do
			if MainFrame.scrollFrame.selectedName == MainFrame.scrollFrame.lines[i]._name then		
				if ns.LootOpened then
					MainFrame.scrollFrame.lines[i].giveIt:Show()
				end
				
				local r, g, b, rs, gs, bs = ns:UnpackFilterIndexColor(MainFrame.scrollFrame.lines[i].filterIndex, MainFrame.scrollFrame.lines[i]._rollType)

				ns.CustomSetTexture(MainFrame.scrollFrame.lines[i].bg, rs, gs, bs, 0.4)
			end
		end
	end
end

function ns:HideGiveItButtons()
	for i=1, #MainFrame.scrollFrame.lines do
		MainFrame.scrollFrame.lines[i].giveIt:Hide()
		
		local r, g, b, rs, gs, bs = ns:UnpackFilterIndexColor(MainFrame.scrollFrame.lines[i].filterIndex, MainFrame.scrollFrame.lines[i]._rollType)
		
		if MainFrame.scrollFrame.selectedName == MainFrame.scrollFrame.lines[i]._name then		
			ns.CustomSetTexture(MainFrame.scrollFrame.lines[i].bg, rs, gs, bs, 0.4)
		else
			ns.CustomSetTexture(MainFrame.scrollFrame.lines[i].bg, r,g,b,0.2)
		end
	end
end

function ns:UpdateColoredFilterSettings()
	ns:HideGiveItButtons()
	ns:ShowGiveItButtons()
end

function ns:UnpackFilterIndexColor(filterIndex, rollType)
	local r, g, b, rs, gs, bs
	local colorType = 'defaulе'
	
	r = 0.8
	g = 0.8
	b = 0.8
	rs = 160/255
	gs = 211/255
	bs = 242/255
	
	--[==[
		
		-1 - Unknown type filter
		0  - Ignore Item / red color
		1  - Need allowed / greed color
		2  - nil
		3  - Offspec allowed / green color
		4  - Need+Offspec / green color
		5  - Transmog allowed
		6  - Need + Transmog
		7  - nil
		8  - Offspec + Transmog 
		9  - All allowed
		
	]==]
	
	if ns.db.ColoredRollFilter then
		local colorType = 'defaul'
		
		if filterIndex == -1 or rollType == 'pass' then 
			colorType = 'defaul'
			
		elseif filterIndex == 9 then
			colorType = 'green'
		elseif rollType == 'need' or rollType == 'upgrade' then
			if filterIndex == 1 or filterIndex == 4 or filterIndex == 6 then
				colorType = 'green'
			else
				colorType = 'red'
			end
		elseif rollType == 'offspec' then
			if filterIndex == 3 or filterIndex == 4 or filterIndex == 8 then
				colorType = 'green'
			else
				colorType = 'red'
			end
		elseif rollType == 'transmog' then
			if filterIndex == 5 or filterIndex == 6 or filterIndex == 8 then
				colorType = 'green'
			else
				colorType = 'red'
			end	
		end
		
		
		if colorType == 'red' then
			r = 0.6
			g = 0.3
			b = 0.3
			
			rs = 255/255
			gs = 76/255
			bs = 76/255
		elseif colorType == 'green' then
			r = 0.3
			g = 0.6
			b = 0.3
			
			rs = 76/255
			gs = 255/255
			bs = 76/255	
		end
	end

	return r, g, b, rs, gs, bs
end

local sortingTable = {}
local pre_sortingTable = {}
local totalListRools = {}

local lastRollID = nil
local sortFuncs = {
	[1] = function(x, y) -- alphabetic sort A > Z
		return x > y
	end,
	[2] = function(x, y) -- Roll type 1 > 5
		
		local x1 = activeLootData[lastRollID].rolls[x].button
		local y1 = activeLootData[lastRollID].rolls[y].button
		
		if x1 == 0 then x1 = 10 end
		if y1 == 0 then y1 = 10 end
		
		return x1 < y1
	end,
	
	[3] = function(x, y) -- Roll numver 1 < 99
		local x1 = activeLootData[lastRollID].rolls[x].rand
		local y1 = activeLootData[lastRollID].rolls[y].rand
		
		return x1 > y1
	end,
	
	[4] = function(x, y)
		local x1 = activeLootData[lastRollID].rolls[x].ilvl
		local y1 = activeLootData[lastRollID].rolls[y].ilvl
		
		return x1 > y1	
	end,
	
	[5] = function(x,y) -- alphabetic sort A < Z
		return x < y
	end,
	[6] = function(x, y) -- Roll type 1 < 5
	--	print('TSort', activeLootData[lastRollID], x, y)
	--	print('TSort2', activeLootData[lastRollID].rolls[x], activeLootData[lastRollID].rolls[y])
	--	print('TSort3', activeLootData[lastRollID].rolls[x].button, activeLootData[lastRollID].rolls[y].button)
		
		local x1 = activeLootData[lastRollID].rolls[x].button
		local y1 = activeLootData[lastRollID].rolls[y].button
	
		if x1 == 0 then x1 = 10 end
		if y1 == 0 then y1 = 10 end
		
		return x1 > y1
	end,
	[7] = function(x, y) -- Roll numver 1 > 99
		local x1 = activeLootData[lastRollID].rolls[x].rand
		local y1 = activeLootData[lastRollID].rolls[y].rand
		
		return x1 < y1
	end,
	
	[8] = function(x, y)
		local x1 = activeLootData[lastRollID].rolls[x].ilvl
		local y1 = activeLootData[lastRollID].rolls[y].ilvl
		
		return x1 < y1	
	end,
}

function ns:GetSortingTableLen()
	return #sortingTable
end

function ns:GetTotalListRollsLen()
	return #totalListRools
end

function ns.GetPlayerRolledInfo(index)
	
	local name = totalListRools[index]
	local lootData = activeLootData[MainFrame.setCurrentRollID]
	local data = lootData.rolls[name]
	local coloredName = nil
	
	if data.classID then
		coloredName = ns.GetColoredNameByID(name, data.classID)
	end
	
	return name, data, coloredName
end

function ns.UpdateScrollLines_ItemGained(link, frame, classID, specID)
--	ns.print('T', 'UpdateScrollLines_ItemGained', link, frame)
	
	local name, link1, quality, iLevel, reqLevel, class, subclass, maxStack, equipSlot, texture, vendorPrice = GetItemInfo(link)				
	if link then
		frame:Show()
		frame.link = link
		frame.classID = classID
		frame.specID = specID
		frame.icon:SetTexture(texture)
	else
		frame.link = nil
		frame.classID = nil
		frame.specID = nil
		frame:Hide()	
	end		
end

function ns:UpdateScrollLines()
	local numLines = 0
	local rollID = MainFrame.setCurrentRollID
	
	local curTableLen = #sortingTable

	if MainFrame.scrollFrame.lastTableLen ~= curTableLen then	
		MainFrame.scrollFrame.tempFrom = 1
	end
	
	for i=1, #MainFrame.scrollFrame.lines do
		MainFrame.scrollFrame.lines[i]:Hide()
	end
	
	for i=MainFrame.scrollFrame.tempFrom, curTableLen do

		numLines = numLines + 1
		
		
		local name = sortingTable[i]
		local data = activeLootData[rollID].rolls[name]
		
		MainFrame.scrollFrame.lines[numLines]._name = name
		MainFrame.scrollFrame.lines[numLines]._rollID = rollID
		MainFrame.scrollFrame.lines[numLines]._link = activeLootData[rollID].link
		MainFrame.scrollFrame.lines[numLines].filterIndex = data.filterIndex
		MainFrame.scrollFrame.lines[numLines].index:SetText(i)
		
		if ns:GetPlayerFullName() ==  name then
			MainFrame.scrollFrame.lines[numLines].rollTypeOverlay:Show()
		else
			MainFrame.scrollFrame.lines[numLines].rollTypeOverlay:Hide()
		end
		--[==[
		local tName = Ambiguate(name, "none")
		local class, classFileName = UnitClass(tName)
		local color = unknownColor2
		if classFileName then
			color = RAID_CLASS_COLORS[classFileName] and RAID_CLASS_COLORS[classFileName].colorStr or unknownColor2
		end	
	
		MainFrame.scrollFrame.lines[numLines].name:SetText('|c'..color..tName..'|r')
		]==]
		
	--	print("T", data.classID, name)
		
		if data.classID then
			MainFrame.scrollFrame.lines[numLines].name:SetText(ns.GetColoredNameByID(name, data.classID))
		else
			MainFrame.scrollFrame.lines[numLines].name:SetText(ns.GetColoredName(name))
		end
	
	
		local texture, tC1, tC2, tC3, tC4 = ns.GetSpecClassString(data.classID, data.specID)
		MainFrame.scrollFrame.lines[numLines].classOrSpecIcon:SetTexture(texture)	
		if tC1 and tC2 and tC3 and tC4 then
			MainFrame.scrollFrame.lines[numLines].classOrSpecIcon:SetTexCoord(tC1, tC2, tC3, tC4)	
		else
			MainFrame.scrollFrame.lines[numLines].classOrSpecIcon:SetTexCoord(0.07, 0.93, 0.07, 0.93)	
		end
		
		local buttonName, buttonTip, buttonArg = ns:GetButtonIDInfo(tonumber(data.button))
		local color = rollTypeColors[buttonTip] or ''
		
		MainFrame.scrollFrame.lines[numLines]._rollType = buttonTip
		MainFrame.scrollFrame.lines[numLines]._buttonID = data.button
		MainFrame.scrollFrame.lines[numLines]._randIndex = data.rand
		
		MainFrame.scrollFrame.lines[numLines].rollType:SetText( color.. ( buttonName or ' button'..data.button ) )
		MainFrame.scrollFrame.lines[numLines].rang:SetText(data.rang)
		MainFrame.scrollFrame.lines[numLines].rollNumber:SetText( (data.button == 0 or data.rand == -1 ) and '' or data.rand)	
		
		MainFrame.scrollFrame.lines[numLines].itemLvl:SetText( data.ilvl == 0 and '' or data.ilvl )
		
		if data.note and data.note ~= '' then
			MainFrame.scrollFrame.lines[numLines].note:SetText('...!')
			MainFrame.scrollFrame.lines[numLines].noteOverlay:Show()
			MainFrame.scrollFrame.lines[numLines].noteOverlay.text = data.note
		else
			MainFrame.scrollFrame.lines[numLines].note:SetText('')
			MainFrame.scrollFrame.lines[numLines].noteOverlay:Hide()
			MainFrame.scrollFrame.lines[numLines].noteOverlay.text = nil
		end
		
		MainFrame.scrollFrame.lines[numLines]:Show()
		
		if data.item1 and data.item1 ~= 'NONE' then			
			ns.QueueForItemInfo(data.item1, 'UpdateScrollLines_ItemGained', MainFrame.scrollFrame.lines[numLines].items1, data.classID, data.specID)	
		else
			MainFrame.scrollFrame.lines[numLines].items1:Hide()			
		end
		
		if data.item2 and data.item2 ~= 'NONE' then
			ns.QueueForItemInfo(data.item2, 'UpdateScrollLines_ItemGained', MainFrame.scrollFrame.lines[numLines].items2, data.classID, data.specID)	
		else
			MainFrame.scrollFrame.lines[numLines].items2:Hide()
		end
		
		if data.item3 and data.item3 ~= 'NONE' then
			ns.QueueForItemInfo(data.item3, 'UpdateScrollLines_ItemGained', MainFrame.scrollFrame.lines[numLines].items3, data.classID, data.specID)	
		else
			MainFrame.scrollFrame.lines[numLines].items3:Hide()
		end
		
		if not MainFrame.scrollFrame.lines[numLines+1] then 
			break
		end
	end
	
--	ns.print('T', 'UpdateScrollLines', 'Show', numLines,'lines')
	
	ns:UpdateColoredFilterSettings()
end

local debug_UpdateRollsListForRollID_timeout = -1

function ns:UpdateRollsListForRollID(rollID)
	
	for i=1, #sortingButtons do
		local button = sortingButtons[i]
		
		if MainFrame.sorting and button.sorting == MainFrame.sorting then
			button.arrow:SetArrowUp()
		elseif MainFrame.sorting and button.revsorting == MainFrame.sorting then
			button.arrow:SetArrowDown()
		else
			button.arrow:SetArrowDisabled()
		end		
	end
	
	if MainFrame.setCurrentRollID and activeLootData[MainFrame.setCurrentRollID] then	
		
	else 
		MainFrame.setCurrentRollID = activeLoot[1] or rollID
		MainFrame.scrollFrame.lastTableLen = -1
	end
	
	if debug_UpdateRollsListForRollID_timeout < GetTime() then
		debug_UpdateRollsListForRollID_timeout = GetTime() + 1
		ns.dprint('UpdateRollsListForRollID', 'current:', MainFrame.setCurrentRollID, 'rollID:',rollID)
	end
	
	local hideTab = true
	for i=1, #MainFrame.icons do
		MainFrame.icons[i].selectedTab = nil
		
		if MainFrame.icons[i].rollID == MainFrame.setCurrentRollID then
			hideTab = false
			MainFrame.selectTab:SetPoint('TOP', MainFrame.icons[i], 'TOP', 0, 5)			
			MainFrame.icons[i].selectedTab = MainFrame.selectTab		
			local link = activeLootData[MainFrame.icons[i].rollID].link	
			ns.QueueForItemInfo(link, 'UpdateIconGrid_ItemGained', MainFrame.icons[i])	
			break
		end
	end
	
	if hideTab then
		MainFrame.selectTab:Hide()
	else
		MainFrame.selectTab:Show()
	end
	
	if MainFrame.setCurrentRollID ~= rollID then
--		ns.print('T', 'Not same rollID', 'Skip for upadte')
		return 
	end
	
	for i=1, #MainFrame.scrollFrame.lines do		
		MainFrame.scrollFrame.lines[i]:Hide()
	end
--	ns.print('T', 'Hide lines')

	
	if activeLootData[rollID] then
	
--		ns.print('T', 'Sorting')
		wipe(sortingTable)
		wipe(pre_sortingTable)
		wipe(totalListRools)
		
		lastRollID = rollID

		for name, data in pairs( activeLootData[rollID].rolls ) do		
			
			totalListRools[#totalListRools+1] = name
		--	print('T', name,#sortingTable+1)
			
			if name then
				if ns.db.showPass then
					local dir = data.button
					if data.button == 0 then
						dir = 10
					end
					
					if not pre_sortingTable[dir] then pre_sortingTable[dir] = {} end
					pre_sortingTable[dir][#pre_sortingTable[dir]+1] = name
				elseif not ns.db.showPass and data.button ~= 0 then
					local dir = data.button
					
					if not pre_sortingTable[dir] then pre_sortingTable[dir] = {} end
					pre_sortingTable[dir][#pre_sortingTable[dir]+1] = name
				end
			end
		end
		
		for buttonID = 1, 10 do
			local data = pre_sortingTable[buttonID]
	--	for buttonID, data in pairs(pre_sortingTable) do
			
	--		print('T', buttonID, data)
			
			if data and #data > 0 then		
				table_sort(data, sortFuncs[MainFrame.sorting or default_sorting_methode])
				
				for index = 1, #data do			
					sortingTable[#sortingTable + 1] = data[index]
				end
			end
		end
		
		ns:UpdateScrollLines()

	end
end

function ns:UpdateAmountRollID(rollID)

	if ns:IsMasterLooter() then
		local frame = GetUsedIconTab(rollID)
		local amount = 0
		
		if frame then			
			for name, data in pairs( activeLootData[rollID].rolls ) do	
				if name then
					if data.button ~= 0 then
						amount = amount + 1
					end
				end
			end
			
			frame.addFrame.text:SetText(amount)
		end
	end	
end

local pendingForShowing = {}
local currentBids = {}
local ignoreBids = {} -- bidsGuid = timeout
local timeoutBids = {}

function ns:GetCurrentLootSessionGUID()
	local guid = nil
	
	for rollID, data in pairs(pendingForShowing) do	
		if data and data.guid then
			guid = data.guid
		end
	end
	
	for rollID, data in pairs(activeLootData) do	
		if data and data.guid then
			guid = data.guid
		end
	end
	
	return guid
end

function ns:IsIgnoredGUID(guid)
	if ignoreBids[guid] then
		if ignoreBids[guid] > GetTime() then
--			ns.print('T', 'Ignore Bids', guid, ignoreBids[guid]-GetTime(), 's left')
			return true
		else
			ignoreBids[guid] = nil
--			ns.print('T', 'Ignore Bids', guid, 'remove ignoring')
		end
	end
	
	return false
end

function ns:IsNotTimeOutGUID()
	local guid = ns:GetCurrentLootSessionGUID()
	
	if guid and timeoutBids[guid] then
	
		return false
	end
	
	return true
end

function ns:AddToTimeOutCurrentLootBids() -- закрытие кнопкой основное меню

	wipe(currentBids)
	
	for rollID, data in pairs(pendingForShowing) do	
		if data and data.guid and not currentBids[data.guid] then
			currentBids[data.guid] = true
		end
	end

	for rollID, data in pairs(activeLootData) do	
		if data and data.guid and not currentBids[data.guid] then
			currentBids[data.guid] = true
		end
	end
	
	for guid in pairs(currentBids) do
		timeoutBids[guid] = GetTime()
	end
end

function ns:ClearTimeoutLootBids()	
	for guid, timeout in pairs(timeoutBids) do
		if timeout + 180 < GetTime() then
			timeoutBids[guid] = nil
			ns.ClearRollIDs(guid)
		end
	end
end

function ns:AddToIgnoreCurrentLootBids() -- вход/лив из рейда. чистка данных
	wipe(currentBids)
	
	for rollID, data in pairs(pendingForShowing) do	
		if data and data.guid and not currentBids[data.guid] then
			currentBids[data.guid] = true
		end
	end

	for rollID, data in pairs(activeLootData) do	
		if data and data.guid and not currentBids[data.guid] then
			currentBids[data.guid] = true
		end
	end
	
	for guid in pairs(currentBids) do
		ignoreBids[guid] = GetTime() + 60
		ns.ClearRollIDs(guid)
	end
end

function ns:ShowLootRolls(info, author)
	
	if ns:IsMasterLooter() then 
	--	if not ns:IsInstanceIgnored_Server() then return end
	else
	--	if not ns:IsInstanceIgnored_Client() then return end
	end

	local lootData = { strsplit('\n', info) }
	
	local guid = lootData[1]
	local index = lootData[2]
	local link = lootData[3]
	
	local button 
	if ns.testMode then
		button = fastrandom(0, 4)
		if button == 5 then
			button = 0
		end
	else
		button = lootData[4] and tonumber(lootData[4])
	end
		
	local item1 = lootData[5]
	local item2 = lootData[6]
	local item3 = lootData[14]
	
	local rang = lootData[7]
	local note = lootData[8]
	
	if note and note == 'No note set' then
		note = ''
	end
	
	local ilvl 
	if ns.testMode then
		ilvl = fastrandom(100, 200)
	else
		ilvl = lootData[9] and tonumber(lootData[9]) or 0
	end
	
	local classID = lootData[11] and tonumber(lootData[11]) or nil
	local specID = lootData[12] and tonumber(lootData[12]) or nil
	
	local filterIndex 
	
	if ns.testMode then
		filterIndex = fastrandom(-1, 9)
		--[==[
		-1 - Unknown type filter
		0  - Ignore Item / red color
		1  - Need/offspec allowd / greed color
		2  - nil
		3  - Offspec allowed / green color
		4  - Need+Offspec / green color
		5  - Transmog allowed
		6  - Need + Transmog
		7  - nil
		8  - Offspec + Tansmog 
		9  - All allowed
		]==]
		
		if ( filterIndex == 2 ) or ( filterIndex == 7 ) then
			filterIndex = -1
		end
	else
		filterIndex = lootData[13] and tonumber(lootData[13]) or -1
	end
	
--	print('T2', author, filterIndex)
	
	if button == 0 and not ns.ShowPass then return end
	if ns:IsIgnoredGUID(guid) then return end
	
	local rollID = guid..'\n'..index..'\n'..link
	
--	ns.print('T', 'ShowLootRolls', author, unpack(lootData))
	local buttonName, buttonTip, buttonArg = ns:GetButtonIDInfo(tonumber(button))
	
	local roll
	if ns.clienRollDice then
		if ns.testMode then
			roll = fastrandom(1,100)
		else
			roll = lootData[10] or -1
		end
	end
	
	if not activeLootData[rollID] then

		if not pendingForShowing[rollID] then
			pendingForShowing[rollID] = { link = link, guid = guid, index = index, rolls = { } }
		end
		
		if not pendingForShowing[rollID].rolls[author] then
			pendingForShowing[rollID].rolls[author] = { }			
			pendingForShowing[rollID].rolls[author].button = button
			pendingForShowing[rollID].rolls[author].item1 = item1 
			pendingForShowing[rollID].rolls[author].item2 = item2
			pendingForShowing[rollID].rolls[author].item3 = item3
			pendingForShowing[rollID].rolls[author].rang = rang
			pendingForShowing[rollID].rolls[author].note = note
			pendingForShowing[rollID].rolls[author].ilvl = ilvl
			pendingForShowing[rollID].rolls[author].rand = buttonArg == 'roll' and roll or -1
			pendingForShowing[rollID].rolls[author].classID = classID
			pendingForShowing[rollID].rolls[author].specID = specID
			pendingForShowing[rollID].rolls[author].filterIndex = filterIndex
		end
	else	
		if not activeLootData[rollID].rolls[author] then
			activeLootData[rollID].rolls[author] = { }			
			activeLootData[rollID].rolls[author].button = button
			activeLootData[rollID].rolls[author].item1 = item1 
			activeLootData[rollID].rolls[author].item2 = item2
			activeLootData[rollID].rolls[author].item3 = item3
			activeLootData[rollID].rolls[author].rang = rang
			activeLootData[rollID].rolls[author].note = note
			activeLootData[rollID].rolls[author].ilvl = ilvl
			activeLootData[rollID].rolls[author].rand = buttonArg == 'roll' and roll or -1
			activeLootData[rollID].rolls[author].classID = classID
			activeLootData[rollID].rolls[author].specID = specID
			activeLootData[rollID].rolls[author].filterIndex = filterIndex
		end	
	end
	
	if ns.clienRollDice then
	
	elseif ns:IsMasterLooter() and button ~= 0 then
		local msg = rollID..'\n'..author..'\n'..fastrandom(1,100)
		
		ns.print('T', 'Send roll dice', author, msg)
		
		ns:SendMessage(msg, ns.channelNameLootRollDice)	
	end
	
	ns:UpdateRollsListForRollID(rollID)
	ns:UpdateAmountRollID(rollID)
end

function ns:UpdateRollDice(info, author)
	local lootData = { strsplit('\n', info) }
	
	local guid   = lootData[1]
	local index  = lootData[2]
	local link   = lootData[3]
	local player = lootData[4]
	local rand	 = lootData[5]
	
	if ns:IsIgnoredGUID(guid) then return end
	
	local rollID = guid..'\n'..index..'\n'..link
	
	ns.print('T', 'Gain roll dice', player, rand)
	ns.print('T', 'Author is ML', author, ns:IsPlayerMasterLooter(author) )
	
	if author == 'TESTER_MODE' then
	if not ns.testMode and not ns:IsPlayerMasterLooter(author) then return end
	end
	
	if not activeLootData[rollID] then
		if not pendingForShowing[rollID] then
		--	ns.dprint('T', 'Cant find rollID', rollID, '. pendingForShowing')
		elseif not pendingForShowing[rollID].rolls[player] then	
		--	ns.dprint('T', 'Cant find', player, 'of roll. pendingForShowing')
		else
			pendingForShowing[rollID].rolls[player].rand = tonumber(rand)			
		end
	else
		if not activeLootData[rollID].rolls[player] then		
		--	ns.print('T', 'Cant find', player, 'of roll. activeLootData')
		else
			activeLootData[rollID].rolls[player].rand = tonumber(rand)
		end
	end
	
	ns:UpdateRollsListForRollID(rollID)
	ns:UpdateAmountRollID(rollID)
end

function ns:ShowLootRollsItemsList(info, author)
	
--	ns.print('T', 'ShowLootRollsItemsList', author)
	
	local lootData = { strsplit('\n', info) }
	local guid = lootData[1]
	
	if ns:IsIgnoredGUID(guid) then return end
	
	for i=2, #lootData do 
		if lootData[i] ~= 'NONE' then
	--		ns.print('T', i-1, lootData[i])	
			
			local index = i-1
			local link = lootData[i]
			local rollID = guid..'\n'..index..'\n'..link
			
	
			if not activeLootData[rollID] then
				activeLoot[#activeLoot+1] = rollID
				
				if pendingForShowing[rollID] then					
					activeLootData[rollID] = { link = link, guid = guid, index = index, rolls = pendingForShowing[rollID].rolls }
					pendingForShowing[rollID] = nil
				else			
					activeLootData[rollID] = { link = link, guid = guid, index = index, rolls = {} }
				end
			end
			
			UpdateIconGrid()
		end
	end
end

function ns:GlowRollTypeUpdate(rollID, author)

	for i=1, #MainFrame.scrollFrame.lines do
		
		local name = MainFrame.scrollFrame.lines[i]._name
		local rollID_temp = MainFrame.scrollFrame.lines[i]._rollID
		
		if name == author and rollID_temp == rollID then
			MainFrame.scrollFrame.lines[i]:Glow()
		--	print('T', i,  MainFrame.scrollFrame.lines[i]._name, MainFrame.scrollFrame.lines[i]._rollID)
		end
	end
end

function ns:UpdateRollType_OnSend(rollID, buttonID)			
--	print('Set to', buttonID, rollID) 
	local guid, index, link = strsplit('\n', rollID)
	
	local filterIndex = 0
	local enableNeed, enableOffspec, enableTransmog = ns.CheckItemFilter(link)
	local enableUpgrade = enableNeed
	
	if enableNeed then filterIndex = filterIndex + 1 end
	if enableOffspec then filterIndex = filterIndex + 3 end
	if enableTransmog then filterIndex = filterIndex + 5 end
	
	local msg = rollID..'\n'..buttonID..'\n'..filterIndex
	
	ns:SendMessage(msg, ns.channelNameLootChangeRoll)				
end

function ns.UpdateRollType_OnChange(info, author)

--	print('Get RollChange', info)
	
	local lootData = { strsplit('\n', info) }
	local guid = lootData[1]
	local index = lootData[2]
	local link = lootData[3]
	local rollType = lootData[4] and tonumber(lootData[4]) or 0
	local filterIndex = lootData[5] and tonumber(lootData[5]) or -2
	
	local rollID = guid..'\n'..index..'\n'..link
	
	if ns:IsIgnoredGUID(guid) then return end
	
	if activeLootData[rollID] then	
		if activeLootData[rollID].rolls[author] then	
			
			local sendRoll = activeLootData[rollID].rolls[author].rand == -1
			
			activeLootData[rollID].rolls[author].button = rollType
			
			if filterIndex and filterIndex ~= -2 then
				activeLootData[rollID].rolls[author].filterIndex = filterIndex
			end
			
		--	print('T', author, activeLootData[rollID].rolls[author].button)
		--	print('T', author, activeLootData[rollID].rolls[author].filterIndex)
			
			if sendRoll and rollType ~= 0 then
				if ns:IsMasterLooter() then
					local msg = rollID..'\n'..author..'\n'..fastrandom(1,100)
					
		--			print('T', 'Send roll if no roll before')
					
					ns:SendMessage(msg, ns.channelNameLootRollDice)	
				end		
			end
			
		end
		
		ns:UpdateRollsListForRollID(rollID)
		ns:GlowRollTypeUpdate(rollID, author)
		ns:UpdateAmountRollID(rollID)
	end	
end

local function ClearRollIDs(guid)	
	for i=1, #activeLoot do
		local rollID = activeLoot[i]	
		if pendingForShowing[rollID] then
			pendingForShowing[rollID] = nil
		end
		if activeLootData[rollID] and activeLootData[rollID].guid == guid then			
			table.remove(activeLoot, i)
			activeLootData[rollID] = nil
			timeoutBids[rollID] = nil

			for i=1, #MainFrame.icons do
			
				if MainFrame.icons[i].rollID == rollID then
					MainFrame.icons[i].rollID = nil
					MainFrame.icons[i]:Hide()
				end
			end
	
			ClearRollIDs(guid)
			return 
		end
	end
	
	MainFrame.setCurrentRollID = nil
	MainFrame.scrollFrame.lastTableLen = -1
	
	UpdateIconGrid()
end
ns.ClearRollIDs = ClearRollIDs

local function ClearOnlyRollID(rollID_del)
	for i=1, #activeLoot do
		local rollID = activeLoot[i]	
		
		if rollID_del == rollID then			
			if pendingForShowing[rollID] then
				pendingForShowing[rollID] = nil
			end
			if activeLootData[rollID] then			
				table.remove(activeLoot, i)
				activeLootData[rollID] = nil
				timeoutBids[rollID] = nil
				
				if MainFrame.setCurrentRollID == rollID then			
					MainFrame.setCurrentRollID = nil --activeLoot[i]
				end
			--[==[
				for i=1, #MainFrame.icons do				
					if MainFrame.icons[i].rollID == rollID then
						MainFrame.icons[i].rollID = nil
						MainFrame.icons[i]:Hide()
					end
				end
				]==]
			end
			break
		end
	end
	
	MainFrame.scrollFrame.lastTableLen = -1
	UpdateIconGrid()
end
ns.ClearOnlyRollID = ClearOnlyRollID

function ns:HideLootRollsItemsList(info, author)
--	ns.print('T', 'HideLootRollsItemsList', author)
	local lootData = { strsplit('\n', info) }
	
	if #lootData == 1 then	
		ClearRollIDs(lootData[1])	
	elseif #lootData == 3 then
		local guid = lootData[1]
		local index = lootData[2]
		local link = lootData[3]
		
	--	print(guid, index, link)
	--	ns.print('T', unpack(lootData))
		
		ClearOnlyRollID(info)
	end
end

function ns.Debug_ClearRollIDByIcon(index)
	
	local me = ns:GetPlayerFullName()
	local rollID = activeLoot[index]
	
	ns:HideLootRollsItemsList(rollID, me)
end

do
	local OUT_RAID = "out"
	local IN_RAID = 'in'
	
	local inRaid = OUT_RAID
	
	events:RegisterEvent("GROUP_ROSTER_UPDATE")

	function events:GROUP_ROSTER_UPDATE()
		if not IsInRaid() and not IsInGroup() then
			if inraid == IN_RAID then
				inraid = OUT_RAID
				ns.dprint("Left from raid - Clear current bids")

				ns:AddToIgnoreCurrentLootBids()
			end
		elseif IsInRaid() or IsInGroup() then
			if inraid == OUT_RAID then
				inraid = IN_RAID
				ns.dprint("Join to raid - Clear current bids")
				
				ns:AddToIgnoreCurrentLootBids()
			end
		end
	end
end